# coding: utf-8

"""
TrueFoundry ML API

No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

import re  # noqa: F401
import warnings
from typing import Optional

from truefoundry.ml._autogen.client.api_client import ApiClient
from truefoundry.ml._autogen.client.api_response import ApiResponse
from truefoundry.ml._autogen.client.exceptions import (  # noqa: F401
    ApiTypeError,
    ApiValueError,
)
from truefoundry.ml._autogen.client.models.create_experiment_request_dto import (
    CreateExperimentRequestDto,
)
from truefoundry.ml._autogen.client.models.create_experiment_response_dto import (
    CreateExperimentResponseDto,
)
from truefoundry.ml._autogen.client.models.experiment_id_request_dto import (
    ExperimentIdRequestDto,
)
from truefoundry.ml._autogen.client.models.experiment_response_dto import (
    ExperimentResponseDto,
)
from truefoundry.ml._autogen.client.models.get_experiment_response_dto import (
    GetExperimentResponseDto,
)
from truefoundry.ml._autogen.client.models.list_colums_response_dto import (
    ListColumsResponseDto,
)
from truefoundry.ml._autogen.client.models.list_experiments_response_dto import (
    ListExperimentsResponseDto,
)
from truefoundry.ml._autogen.client.models.set_experiment_tag_request_dto import (
    SetExperimentTagRequestDto,
)
from truefoundry.ml._autogen.client.models.update_experiment_request_dto import (
    UpdateExperimentRequestDto,
)
from truefoundry.pydantic_v1 import (
    StrictInt,
    StrictStr,
    validate_arguments,
)


class ExperimentsApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client

    @validate_arguments
    def create_experiment_post(
        self, create_experiment_request_dto: CreateExperimentRequestDto, **kwargs
    ) -> CreateExperimentResponseDto:  # noqa: E501
        """(Deprecated) Create Experiment  # noqa: E501

        Create Experiment  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.create_experiment_post(create_experiment_request_dto, async_req=True)
        >>> result = thread.get()

        :param create_experiment_request_dto: (required)
        :type create_experiment_request_dto: CreateExperimentRequestDto
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: CreateExperimentResponseDto
        """
        kwargs["_return_http_data_only"] = True
        if "_preload_content" in kwargs:
            message = "Error! Please call the create_experiment_post_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return self.create_experiment_post_with_http_info(
            create_experiment_request_dto, **kwargs
        )  # noqa: E501

    @validate_arguments
    def create_experiment_post_with_http_info(
        self, create_experiment_request_dto: CreateExperimentRequestDto, **kwargs
    ) -> ApiResponse:  # noqa: E501
        """(Deprecated) Create Experiment  # noqa: E501

        Create Experiment  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.create_experiment_post_with_http_info(create_experiment_request_dto, async_req=True)
        >>> result = thread.get()

        :param create_experiment_request_dto: (required)
        :type create_experiment_request_dto: CreateExperimentRequestDto
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(CreateExperimentResponseDto, status_code(int), headers(HTTPHeaderDict))
        """

        warnings.warn(
            "POST /api/2.0/mlflow/experiments/create is deprecated.", DeprecationWarning
        )

        _params = locals()

        _all_params = ["create_experiment_request_dto"]
        _all_params.extend(
            [
                "async_req",
                "_return_http_data_only",
                "_preload_content",
                "_request_timeout",
                "_request_auth",
                "_content_type",
                "_headers",
            ]
        )

        # validate the arguments
        for _key, _val in _params["kwargs"].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method create_experiment_post" % _key
                )
            _params[_key] = _val
        del _params["kwargs"]

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get("_headers", {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params["create_experiment_request_dto"] is not None:
            _body_params = _params["create_experiment_request_dto"]

        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get(
            "_content_type",
            self.api_client.select_header_content_type(["application/json"]),
        )
        if _content_types_list:
            _header_params["Content-Type"] = _content_types_list

        # authentication setting
        _auth_settings = ["APIKey"]  # noqa: E501

        _response_types_map = {
            "200": "CreateExperimentResponseDto",
            "422": "HTTPValidationError",
        }

        return self.api_client.call_api(
            "/api/2.0/mlflow/experiments/create",
            "POST",
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get("async_req"),
            _return_http_data_only=_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=_params.get("_preload_content", True),
            _request_timeout=_params.get("_request_timeout"),
            collection_formats=_collection_formats,
            _request_auth=_params.get("_request_auth"),
        )

    @validate_arguments
    def delete_experiment_post(
        self, experiment_id_request_dto: ExperimentIdRequestDto, **kwargs
    ) -> object:  # noqa: E501
        """(Deprecated) Delete Experiment  # noqa: E501

        Delete Experiment  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.delete_experiment_post(experiment_id_request_dto, async_req=True)
        >>> result = thread.get()

        :param experiment_id_request_dto: (required)
        :type experiment_id_request_dto: ExperimentIdRequestDto
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: object
        """
        kwargs["_return_http_data_only"] = True
        if "_preload_content" in kwargs:
            message = "Error! Please call the delete_experiment_post_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return self.delete_experiment_post_with_http_info(
            experiment_id_request_dto, **kwargs
        )  # noqa: E501

    @validate_arguments
    def delete_experiment_post_with_http_info(
        self, experiment_id_request_dto: ExperimentIdRequestDto, **kwargs
    ) -> ApiResponse:  # noqa: E501
        """(Deprecated) Delete Experiment  # noqa: E501

        Delete Experiment  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.delete_experiment_post_with_http_info(experiment_id_request_dto, async_req=True)
        >>> result = thread.get()

        :param experiment_id_request_dto: (required)
        :type experiment_id_request_dto: ExperimentIdRequestDto
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(object, status_code(int), headers(HTTPHeaderDict))
        """

        warnings.warn(
            "POST /api/2.0/mlflow/experiments/delete is deprecated.", DeprecationWarning
        )

        _params = locals()

        _all_params = ["experiment_id_request_dto"]
        _all_params.extend(
            [
                "async_req",
                "_return_http_data_only",
                "_preload_content",
                "_request_timeout",
                "_request_auth",
                "_content_type",
                "_headers",
            ]
        )

        # validate the arguments
        for _key, _val in _params["kwargs"].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method delete_experiment_post" % _key
                )
            _params[_key] = _val
        del _params["kwargs"]

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get("_headers", {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params["experiment_id_request_dto"] is not None:
            _body_params = _params["experiment_id_request_dto"]

        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get(
            "_content_type",
            self.api_client.select_header_content_type(["application/json"]),
        )
        if _content_types_list:
            _header_params["Content-Type"] = _content_types_list

        # authentication setting
        _auth_settings = ["APIKey"]  # noqa: E501

        _response_types_map = {
            "200": "object",
            "422": "HTTPValidationError",
        }

        return self.api_client.call_api(
            "/api/2.0/mlflow/experiments/delete",
            "POST",
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get("async_req"),
            _return_http_data_only=_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=_params.get("_preload_content", True),
            _request_timeout=_params.get("_request_timeout"),
            collection_formats=_collection_formats,
            _request_auth=_params.get("_request_auth"),
        )

    @validate_arguments
    def get_experiment_by_name_get(
        self, experiment_name: StrictStr, **kwargs
    ) -> ExperimentResponseDto:  # noqa: E501
        """(Deprecated) Get Experiment By Name  # noqa: E501

        Get Experiment By Name  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_experiment_by_name_get(experiment_name, async_req=True)
        >>> result = thread.get()

        :param experiment_name: (required)
        :type experiment_name: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: ExperimentResponseDto
        """
        kwargs["_return_http_data_only"] = True
        if "_preload_content" in kwargs:
            message = "Error! Please call the get_experiment_by_name_get_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return self.get_experiment_by_name_get_with_http_info(experiment_name, **kwargs)  # noqa: E501

    @validate_arguments
    def get_experiment_by_name_get_with_http_info(
        self, experiment_name: StrictStr, **kwargs
    ) -> ApiResponse:  # noqa: E501
        """(Deprecated) Get Experiment By Name  # noqa: E501

        Get Experiment By Name  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_experiment_by_name_get_with_http_info(experiment_name, async_req=True)
        >>> result = thread.get()

        :param experiment_name: (required)
        :type experiment_name: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(ExperimentResponseDto, status_code(int), headers(HTTPHeaderDict))
        """

        warnings.warn(
            "GET /api/2.0/mlflow/experiments/get-by-name is deprecated.",
            DeprecationWarning,
        )

        _params = locals()

        _all_params = ["experiment_name"]
        _all_params.extend(
            [
                "async_req",
                "_return_http_data_only",
                "_preload_content",
                "_request_timeout",
                "_request_auth",
                "_content_type",
                "_headers",
            ]
        )

        # validate the arguments
        for _key, _val in _params["kwargs"].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_experiment_by_name_get" % _key
                )
            _params[_key] = _val
        del _params["kwargs"]

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get("experiment_name") is not None:  # noqa: E501
            _query_params.append(("experiment_name", _params["experiment_name"]))

        # process the header parameters
        _header_params = dict(_params.get("_headers", {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # authentication setting
        _auth_settings = ["APIKey"]  # noqa: E501

        _response_types_map = {
            "200": "ExperimentResponseDto",
            "422": "HTTPValidationError",
        }

        return self.api_client.call_api(
            "/api/2.0/mlflow/experiments/get-by-name",
            "GET",
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get("async_req"),
            _return_http_data_only=_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=_params.get("_preload_content", True),
            _request_timeout=_params.get("_request_timeout"),
            collection_formats=_collection_formats,
            _request_auth=_params.get("_request_auth"),
        )

    @validate_arguments
    def get_experiment_get(
        self, experiment_id: StrictStr, **kwargs
    ) -> GetExperimentResponseDto:  # noqa: E501
        """(Deprecated) Get Experiment  # noqa: E501

        Get Experiment  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_experiment_get(experiment_id, async_req=True)
        >>> result = thread.get()

        :param experiment_id: (required)
        :type experiment_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: GetExperimentResponseDto
        """
        kwargs["_return_http_data_only"] = True
        if "_preload_content" in kwargs:
            message = "Error! Please call the get_experiment_get_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return self.get_experiment_get_with_http_info(experiment_id, **kwargs)  # noqa: E501

    @validate_arguments
    def get_experiment_get_with_http_info(
        self, experiment_id: StrictStr, **kwargs
    ) -> ApiResponse:  # noqa: E501
        """(Deprecated) Get Experiment  # noqa: E501

        Get Experiment  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_experiment_get_with_http_info(experiment_id, async_req=True)
        >>> result = thread.get()

        :param experiment_id: (required)
        :type experiment_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(GetExperimentResponseDto, status_code(int), headers(HTTPHeaderDict))
        """

        warnings.warn(
            "GET /api/2.0/mlflow/experiments/get is deprecated.", DeprecationWarning
        )

        _params = locals()

        _all_params = ["experiment_id"]
        _all_params.extend(
            [
                "async_req",
                "_return_http_data_only",
                "_preload_content",
                "_request_timeout",
                "_request_auth",
                "_content_type",
                "_headers",
            ]
        )

        # validate the arguments
        for _key, _val in _params["kwargs"].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_experiment_get" % _key
                )
            _params[_key] = _val
        del _params["kwargs"]

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get("experiment_id") is not None:  # noqa: E501
            _query_params.append(("experiment_id", _params["experiment_id"]))

        # process the header parameters
        _header_params = dict(_params.get("_headers", {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # authentication setting
        _auth_settings = ["APIKey"]  # noqa: E501

        _response_types_map = {
            "200": "GetExperimentResponseDto",
            "422": "HTTPValidationError",
        }

        return self.api_client.call_api(
            "/api/2.0/mlflow/experiments/get",
            "GET",
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get("async_req"),
            _return_http_data_only=_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=_params.get("_preload_content", True),
            _request_timeout=_params.get("_request_timeout"),
            collection_formats=_collection_formats,
            _request_auth=_params.get("_request_auth"),
        )

    @validate_arguments
    def hard_delete_experiment_post(
        self, experiment_id_request_dto: ExperimentIdRequestDto, **kwargs
    ) -> object:  # noqa: E501
        """(Deprecated) Hard Delete Experiment  # noqa: E501

        Hard Delete Experiment  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.hard_delete_experiment_post(experiment_id_request_dto, async_req=True)
        >>> result = thread.get()

        :param experiment_id_request_dto: (required)
        :type experiment_id_request_dto: ExperimentIdRequestDto
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: object
        """
        kwargs["_return_http_data_only"] = True
        if "_preload_content" in kwargs:
            message = "Error! Please call the hard_delete_experiment_post_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return self.hard_delete_experiment_post_with_http_info(
            experiment_id_request_dto, **kwargs
        )  # noqa: E501

    @validate_arguments
    def hard_delete_experiment_post_with_http_info(
        self, experiment_id_request_dto: ExperimentIdRequestDto, **kwargs
    ) -> ApiResponse:  # noqa: E501
        """(Deprecated) Hard Delete Experiment  # noqa: E501

        Hard Delete Experiment  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.hard_delete_experiment_post_with_http_info(experiment_id_request_dto, async_req=True)
        >>> result = thread.get()

        :param experiment_id_request_dto: (required)
        :type experiment_id_request_dto: ExperimentIdRequestDto
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(object, status_code(int), headers(HTTPHeaderDict))
        """

        warnings.warn(
            "POST /api/2.0/mlflow/experiments/hard-delete is deprecated.",
            DeprecationWarning,
        )

        _params = locals()

        _all_params = ["experiment_id_request_dto"]
        _all_params.extend(
            [
                "async_req",
                "_return_http_data_only",
                "_preload_content",
                "_request_timeout",
                "_request_auth",
                "_content_type",
                "_headers",
            ]
        )

        # validate the arguments
        for _key, _val in _params["kwargs"].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method hard_delete_experiment_post" % _key
                )
            _params[_key] = _val
        del _params["kwargs"]

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get("_headers", {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params["experiment_id_request_dto"] is not None:
            _body_params = _params["experiment_id_request_dto"]

        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get(
            "_content_type",
            self.api_client.select_header_content_type(["application/json"]),
        )
        if _content_types_list:
            _header_params["Content-Type"] = _content_types_list

        # authentication setting
        _auth_settings = ["APIKey"]  # noqa: E501

        _response_types_map = {
            "200": "object",
            "422": "HTTPValidationError",
        }

        return self.api_client.call_api(
            "/api/2.0/mlflow/experiments/hard-delete",
            "POST",
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get("async_req"),
            _return_http_data_only=_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=_params.get("_preload_content", True),
            _request_timeout=_params.get("_request_timeout"),
            collection_formats=_collection_formats,
            _request_auth=_params.get("_request_auth"),
        )

    @validate_arguments
    def list_experiment_columns_get(
        self, experiment_id: StrictStr, **kwargs
    ) -> ListColumsResponseDto:  # noqa: E501
        """(Deprecated) List Experiment Columns  # noqa: E501

        List all the columns required for an experiment.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.list_experiment_columns_get(experiment_id, async_req=True)
        >>> result = thread.get()

        :param experiment_id: (required)
        :type experiment_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: ListColumsResponseDto
        """
        kwargs["_return_http_data_only"] = True
        if "_preload_content" in kwargs:
            message = "Error! Please call the list_experiment_columns_get_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return self.list_experiment_columns_get_with_http_info(experiment_id, **kwargs)  # noqa: E501

    @validate_arguments
    def list_experiment_columns_get_with_http_info(
        self, experiment_id: StrictStr, **kwargs
    ) -> ApiResponse:  # noqa: E501
        """(Deprecated) List Experiment Columns  # noqa: E501

        List all the columns required for an experiment.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.list_experiment_columns_get_with_http_info(experiment_id, async_req=True)
        >>> result = thread.get()

        :param experiment_id: (required)
        :type experiment_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(ListColumsResponseDto, status_code(int), headers(HTTPHeaderDict))
        """

        warnings.warn(
            "GET /api/2.0/mlflow/experiments/columns is deprecated.", DeprecationWarning
        )

        _params = locals()

        _all_params = ["experiment_id"]
        _all_params.extend(
            [
                "async_req",
                "_return_http_data_only",
                "_preload_content",
                "_request_timeout",
                "_request_auth",
                "_content_type",
                "_headers",
            ]
        )

        # validate the arguments
        for _key, _val in _params["kwargs"].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method list_experiment_columns_get" % _key
                )
            _params[_key] = _val
        del _params["kwargs"]

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get("experiment_id") is not None:  # noqa: E501
            _query_params.append(("experiment_id", _params["experiment_id"]))

        # process the header parameters
        _header_params = dict(_params.get("_headers", {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # authentication setting
        _auth_settings = ["APIKey"]  # noqa: E501

        _response_types_map = {
            "200": "ListColumsResponseDto",
            "422": "HTTPValidationError",
        }

        return self.api_client.call_api(
            "/api/2.0/mlflow/experiments/columns",
            "GET",
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get("async_req"),
            _return_http_data_only=_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=_params.get("_preload_content", True),
            _request_timeout=_params.get("_request_timeout"),
            collection_formats=_collection_formats,
            _request_auth=_params.get("_request_auth"),
        )

    @validate_arguments
    def list_experiments_get(
        self,
        view_type: Optional[StrictStr] = None,
        max_results: Optional[StrictInt] = None,
        page_token: Optional[StrictStr] = None,
        privacy_type: Optional[StrictStr] = None,
        offset: Optional[StrictInt] = None,
        filter_name: Optional[StrictStr] = None,
        **kwargs,
    ) -> ListExperimentsResponseDto:  # noqa: E501
        """(Deprecated) List Experiments  # noqa: E501

        List Experiments  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.list_experiments_get(view_type, max_results, page_token, privacy_type, offset, filter_name, async_req=True)
        >>> result = thread.get()

        :param view_type:
        :type view_type: str
        :param max_results:
        :type max_results: int
        :param page_token:
        :type page_token: str
        :param privacy_type:
        :type privacy_type: str
        :param offset:
        :type offset: int
        :param filter_name:
        :type filter_name: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: ListExperimentsResponseDto
        """
        kwargs["_return_http_data_only"] = True
        if "_preload_content" in kwargs:
            message = "Error! Please call the list_experiments_get_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return self.list_experiments_get_with_http_info(
            view_type,
            max_results,
            page_token,
            privacy_type,
            offset,
            filter_name,
            **kwargs,
        )  # noqa: E501

    @validate_arguments
    def list_experiments_get_with_http_info(
        self,
        view_type: Optional[StrictStr] = None,
        max_results: Optional[StrictInt] = None,
        page_token: Optional[StrictStr] = None,
        privacy_type: Optional[StrictStr] = None,
        offset: Optional[StrictInt] = None,
        filter_name: Optional[StrictStr] = None,
        **kwargs,
    ) -> ApiResponse:  # noqa: E501
        """(Deprecated) List Experiments  # noqa: E501

        List Experiments  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.list_experiments_get_with_http_info(view_type, max_results, page_token, privacy_type, offset, filter_name, async_req=True)
        >>> result = thread.get()

        :param view_type:
        :type view_type: str
        :param max_results:
        :type max_results: int
        :param page_token:
        :type page_token: str
        :param privacy_type:
        :type privacy_type: str
        :param offset:
        :type offset: int
        :param filter_name:
        :type filter_name: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(ListExperimentsResponseDto, status_code(int), headers(HTTPHeaderDict))
        """

        warnings.warn(
            "GET /api/2.0/mlflow/experiments/list is deprecated.", DeprecationWarning
        )

        _params = locals()

        _all_params = [
            "view_type",
            "max_results",
            "page_token",
            "privacy_type",
            "offset",
            "filter_name",
        ]
        _all_params.extend(
            [
                "async_req",
                "_return_http_data_only",
                "_preload_content",
                "_request_timeout",
                "_request_auth",
                "_content_type",
                "_headers",
            ]
        )

        # validate the arguments
        for _key, _val in _params["kwargs"].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method list_experiments_get" % _key
                )
            _params[_key] = _val
        del _params["kwargs"]

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get("view_type") is not None:  # noqa: E501
            _query_params.append(("view_type", _params["view_type"]))

        if _params.get("max_results") is not None:  # noqa: E501
            _query_params.append(("max_results", _params["max_results"]))

        if _params.get("page_token") is not None:  # noqa: E501
            _query_params.append(("page_token", _params["page_token"]))

        if _params.get("privacy_type") is not None:  # noqa: E501
            _query_params.append(("privacy_type", _params["privacy_type"]))

        if _params.get("offset") is not None:  # noqa: E501
            _query_params.append(("offset", _params["offset"]))

        if _params.get("filter_name") is not None:  # noqa: E501
            _query_params.append(("filter_name", _params["filter_name"]))

        # process the header parameters
        _header_params = dict(_params.get("_headers", {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # authentication setting
        _auth_settings = ["APIKey"]  # noqa: E501

        _response_types_map = {
            "200": "ListExperimentsResponseDto",
            "422": "HTTPValidationError",
        }

        return self.api_client.call_api(
            "/api/2.0/mlflow/experiments/list",
            "GET",
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get("async_req"),
            _return_http_data_only=_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=_params.get("_preload_content", True),
            _request_timeout=_params.get("_request_timeout"),
            collection_formats=_collection_formats,
            _request_auth=_params.get("_request_auth"),
        )

    @validate_arguments
    def restore_experiment_post(
        self, experiment_id_request_dto: ExperimentIdRequestDto, **kwargs
    ) -> object:  # noqa: E501
        """(Deprecated) Restore Experiment  # noqa: E501

        Restore Experiment  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.restore_experiment_post(experiment_id_request_dto, async_req=True)
        >>> result = thread.get()

        :param experiment_id_request_dto: (required)
        :type experiment_id_request_dto: ExperimentIdRequestDto
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: object
        """
        kwargs["_return_http_data_only"] = True
        if "_preload_content" in kwargs:
            message = "Error! Please call the restore_experiment_post_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return self.restore_experiment_post_with_http_info(
            experiment_id_request_dto, **kwargs
        )  # noqa: E501

    @validate_arguments
    def restore_experiment_post_with_http_info(
        self, experiment_id_request_dto: ExperimentIdRequestDto, **kwargs
    ) -> ApiResponse:  # noqa: E501
        """(Deprecated) Restore Experiment  # noqa: E501

        Restore Experiment  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.restore_experiment_post_with_http_info(experiment_id_request_dto, async_req=True)
        >>> result = thread.get()

        :param experiment_id_request_dto: (required)
        :type experiment_id_request_dto: ExperimentIdRequestDto
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(object, status_code(int), headers(HTTPHeaderDict))
        """

        warnings.warn(
            "POST /api/2.0/mlflow/experiments/restore is deprecated.",
            DeprecationWarning,
        )

        _params = locals()

        _all_params = ["experiment_id_request_dto"]
        _all_params.extend(
            [
                "async_req",
                "_return_http_data_only",
                "_preload_content",
                "_request_timeout",
                "_request_auth",
                "_content_type",
                "_headers",
            ]
        )

        # validate the arguments
        for _key, _val in _params["kwargs"].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method restore_experiment_post" % _key
                )
            _params[_key] = _val
        del _params["kwargs"]

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get("_headers", {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params["experiment_id_request_dto"] is not None:
            _body_params = _params["experiment_id_request_dto"]

        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get(
            "_content_type",
            self.api_client.select_header_content_type(["application/json"]),
        )
        if _content_types_list:
            _header_params["Content-Type"] = _content_types_list

        # authentication setting
        _auth_settings = ["APIKey"]  # noqa: E501

        _response_types_map = {
            "200": "object",
            "422": "HTTPValidationError",
        }

        return self.api_client.call_api(
            "/api/2.0/mlflow/experiments/restore",
            "POST",
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get("async_req"),
            _return_http_data_only=_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=_params.get("_preload_content", True),
            _request_timeout=_params.get("_request_timeout"),
            collection_formats=_collection_formats,
            _request_auth=_params.get("_request_auth"),
        )

    @validate_arguments
    def set_experiment_tag_post(
        self, set_experiment_tag_request_dto: SetExperimentTagRequestDto, **kwargs
    ) -> object:  # noqa: E501
        """(Deprecated) Set Experiment Tag  # noqa: E501

        Set Experiment Tag  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.set_experiment_tag_post(set_experiment_tag_request_dto, async_req=True)
        >>> result = thread.get()

        :param set_experiment_tag_request_dto: (required)
        :type set_experiment_tag_request_dto: SetExperimentTagRequestDto
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: object
        """
        kwargs["_return_http_data_only"] = True
        if "_preload_content" in kwargs:
            message = "Error! Please call the set_experiment_tag_post_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return self.set_experiment_tag_post_with_http_info(
            set_experiment_tag_request_dto, **kwargs
        )  # noqa: E501

    @validate_arguments
    def set_experiment_tag_post_with_http_info(
        self, set_experiment_tag_request_dto: SetExperimentTagRequestDto, **kwargs
    ) -> ApiResponse:  # noqa: E501
        """(Deprecated) Set Experiment Tag  # noqa: E501

        Set Experiment Tag  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.set_experiment_tag_post_with_http_info(set_experiment_tag_request_dto, async_req=True)
        >>> result = thread.get()

        :param set_experiment_tag_request_dto: (required)
        :type set_experiment_tag_request_dto: SetExperimentTagRequestDto
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(object, status_code(int), headers(HTTPHeaderDict))
        """

        warnings.warn(
            "POST /api/2.0/mlflow/experiments/set-experiment-tag is deprecated.",
            DeprecationWarning,
        )

        _params = locals()

        _all_params = ["set_experiment_tag_request_dto"]
        _all_params.extend(
            [
                "async_req",
                "_return_http_data_only",
                "_preload_content",
                "_request_timeout",
                "_request_auth",
                "_content_type",
                "_headers",
            ]
        )

        # validate the arguments
        for _key, _val in _params["kwargs"].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method set_experiment_tag_post" % _key
                )
            _params[_key] = _val
        del _params["kwargs"]

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get("_headers", {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params["set_experiment_tag_request_dto"] is not None:
            _body_params = _params["set_experiment_tag_request_dto"]

        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get(
            "_content_type",
            self.api_client.select_header_content_type(["application/json"]),
        )
        if _content_types_list:
            _header_params["Content-Type"] = _content_types_list

        # authentication setting
        _auth_settings = ["APIKey"]  # noqa: E501

        _response_types_map = {
            "200": "object",
            "422": "HTTPValidationError",
        }

        return self.api_client.call_api(
            "/api/2.0/mlflow/experiments/set-experiment-tag",
            "POST",
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get("async_req"),
            _return_http_data_only=_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=_params.get("_preload_content", True),
            _request_timeout=_params.get("_request_timeout"),
            collection_formats=_collection_formats,
            _request_auth=_params.get("_request_auth"),
        )

    @validate_arguments
    def update_experiment_post(
        self, update_experiment_request_dto: UpdateExperimentRequestDto, **kwargs
    ) -> object:  # noqa: E501
        """(Deprecated) Update Experiment  # noqa: E501

        Update Experiment  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.update_experiment_post(update_experiment_request_dto, async_req=True)
        >>> result = thread.get()

        :param update_experiment_request_dto: (required)
        :type update_experiment_request_dto: UpdateExperimentRequestDto
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: object
        """
        kwargs["_return_http_data_only"] = True
        if "_preload_content" in kwargs:
            message = "Error! Please call the update_experiment_post_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return self.update_experiment_post_with_http_info(
            update_experiment_request_dto, **kwargs
        )  # noqa: E501

    @validate_arguments
    def update_experiment_post_with_http_info(
        self, update_experiment_request_dto: UpdateExperimentRequestDto, **kwargs
    ) -> ApiResponse:  # noqa: E501
        """(Deprecated) Update Experiment  # noqa: E501

        Update Experiment  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.update_experiment_post_with_http_info(update_experiment_request_dto, async_req=True)
        >>> result = thread.get()

        :param update_experiment_request_dto: (required)
        :type update_experiment_request_dto: UpdateExperimentRequestDto
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(object, status_code(int), headers(HTTPHeaderDict))
        """

        warnings.warn(
            "POST /api/2.0/mlflow/experiments/update is deprecated.", DeprecationWarning
        )

        _params = locals()

        _all_params = ["update_experiment_request_dto"]
        _all_params.extend(
            [
                "async_req",
                "_return_http_data_only",
                "_preload_content",
                "_request_timeout",
                "_request_auth",
                "_content_type",
                "_headers",
            ]
        )

        # validate the arguments
        for _key, _val in _params["kwargs"].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method update_experiment_post" % _key
                )
            _params[_key] = _val
        del _params["kwargs"]

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get("_headers", {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params["update_experiment_request_dto"] is not None:
            _body_params = _params["update_experiment_request_dto"]

        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get(
            "_content_type",
            self.api_client.select_header_content_type(["application/json"]),
        )
        if _content_types_list:
            _header_params["Content-Type"] = _content_types_list

        # authentication setting
        _auth_settings = ["APIKey"]  # noqa: E501

        _response_types_map = {
            "200": "object",
            "422": "HTTPValidationError",
        }

        return self.api_client.call_api(
            "/api/2.0/mlflow/experiments/update",
            "POST",
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get("async_req"),
            _return_http_data_only=_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=_params.get("_preload_content", True),
            _request_timeout=_params.get("_request_timeout"),
            collection_formats=_collection_formats,
            _request_auth=_params.get("_request_auth"),
        )



from typing import Any, Callable, Dict, List, Optional, Union

from flytekit.core.base_task import Task
from flytekit.core.launch_plan import LaunchPlan
from flytekit.core.schedule import CronSchedule
from flytekit.core.workflow import (
    FuncOut,
    PythonFunctionWorkflow,
    WorkflowFailurePolicy,
)
from flytekit.core.workflow import workflow as flytekit_workflow

from truefoundry.deploy._autogen.models import WorkflowAlert
from truefoundry.pydantic_v1 import BaseModel

TRUEFOUNDRY_LAUNCH_PLAN_NAME = "default"
TRUEFOUNDRY_ALERTS_CONFIG = "tfy_alerts_config"


class ExecutionConfig(BaseModel):
    # I am not using job's Schedule here because:
    # 1. flyte accepts schedule only as a string
    # 2. flyte doesn't support setting concurrency policy
    schedule: str


class TruefoundryConfigStore(BaseModel):
    execution_config_map: Dict[str, ExecutionConfig] = {}

    def reset(self):
        self.execution_config_map = {}

    def get(self, key: str) -> Optional[ExecutionConfig]:
        return self.execution_config_map.get(key)

    def set(self, key: str, value: ExecutionConfig):
        self.execution_config_map[key] = value


truefoundry_config_store = TruefoundryConfigStore()


def workflow(
    _workflow_function: Optional[Callable[..., Any]] = None,
    failure_policy: Optional[WorkflowFailurePolicy] = None,
    on_failure: Optional[Task] = None,
    execution_configs: Optional[List[ExecutionConfig]] = None,
    alerts: Optional[List[WorkflowAlert]] = None,
) -> Union[
    Callable[[Callable[..., FuncOut]], PythonFunctionWorkflow],
    PythonFunctionWorkflow,
    Callable[..., FuncOut],
]:
    """
    This decorator declares a function to be a Flyte workflow. Workflows are declarative entities that construct a DAG
    of tasks using the data flow between tasks.

    Unlike a task, the function body of a workflow is evaluated at serialization-time (aka compile-time). This is
    because while we can determine the entire structure of a task by looking at the function's signature, workflows need
    to run through the function itself because the body of the function is what expresses the workflow structure. It's
    also important to note that, local execution notwithstanding, it is not evaluated again when the workflow runs on
    Flyte.
    That is, workflows should not call non-Flyte entities since they are only run once (again, this is with respect to
    the platform, local runs notwithstanding).

    Example:

    .. literalinclude:: ../../../tests/flytekit/unit/core/test_workflows.py
       :pyobject: my_wf_example

    Again, users should keep in mind that even though the body of the function looks like regular Python, it is
    actually not. When flytekit scans the workflow function, the objects being passed around between the tasks are not
    your typical Python values. So even though you may have a task ``t1() -> int``, when ``a = t1()`` is called, ``a``
    will not be an integer so if you try to ``range(a)`` you'll get an error.

    Please see the :ref:`user guide <cookbook:workflow>` for more usage examples.

    :param _workflow_function: This argument is implicitly passed and represents the decorated function.
    :param failure_policy: Use the options in flytekit.WorkflowFailurePolicy
    :param on_failure: Invoke this workflow or task on failure. The Workflow / task has to match the signature of
         the current workflow, with an additional parameter called `error` Error
    """
    if _workflow_function is None:

        def wrapper(func: Callable[..., Any]) -> Any:
            return workflow(
                _workflow_function=func,
                failure_policy=failure_policy,
                on_failure=on_failure,
                execution_configs=execution_configs,
                alerts=alerts,
            )

        return wrapper

    workflow_object = flytekit_workflow(
        _workflow_function,
        failure_policy=failure_policy,
        on_failure=on_failure,
    )
    execution_configs = execution_configs or []
    if len(execution_configs) > 1:
        raise ValueError("Only one cron execution config is allowed per workflow")

    if execution_configs:
        execution_config = execution_configs[0]
        # flyte maintains this in-memory and uses it while serialization
        LaunchPlan.get_or_create(
            workflow=workflow_object,
            name=TRUEFOUNDRY_LAUNCH_PLAN_NAME,
            schedule=CronSchedule(schedule=execution_config.schedule),
        )
        function_module = _workflow_function.__module__
        function_name = _workflow_function.__name__
        function_path = f"{function_module}.{function_name}"
        truefoundry_config_store.set(function_path, execution_config)

    if alerts:
        truefoundry_config_store.set(TRUEFOUNDRY_ALERTS_CONFIG, alerts)

    return workflow_object



# coding: utf-8

"""
TrueFoundry ML API

No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Optional

from truefoundry.ml._autogen.client.models.artifact_type import ArtifactType
from truefoundry.ml._autogen.client.models.artifact_version_status import (
    ArtifactVersionStatus,
)
from truefoundry.pydantic_v1 import BaseModel, StrictBool, StrictInt, StrictStr, conlist


class ListArtifactVersionsRequestDto(BaseModel):
    """
    ListArtifactVersionsRequestDto
    """

    artifact_id: Optional[StrictStr] = None
    statuses: Optional[conlist(ArtifactVersionStatus)] = None
    max_results: Optional[StrictInt] = None
    offset: Optional[StrictInt] = None
    page_token: Optional[StrictStr] = None
    run_ids: Optional[conlist(StrictStr)] = None
    artifact_types: Optional[conlist(ArtifactType)] = None
    run_steps: Optional[conlist(StrictInt)] = None
    include_internal_metadata: Optional[StrictBool] = False
    include_model_versions: Optional[StrictBool] = False
    __properties = [
        "artifact_id",
        "statuses",
        "max_results",
        "offset",
        "page_token",
        "run_ids",
        "artifact_types",
        "run_steps",
        "include_internal_metadata",
        "include_model_versions",
    ]

    class Config:
        """Pydantic configuration"""

        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> ListArtifactVersionsRequestDto:
        """Create an instance of ListArtifactVersionsRequestDto from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True, exclude={}, exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> ListArtifactVersionsRequestDto:
        """Create an instance of ListArtifactVersionsRequestDto from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return ListArtifactVersionsRequestDto.parse_obj(obj)

        _obj = ListArtifactVersionsRequestDto.parse_obj(
            {
                "artifact_id": obj.get("artifact_id"),
                "statuses": obj.get("statuses"),
                "max_results": obj.get("max_results"),
                "offset": obj.get("offset"),
                "page_token": obj.get("page_token"),
                "run_ids": obj.get("run_ids"),
                "artifact_types": obj.get("artifact_types"),
                "run_steps": obj.get("run_steps"),
                "include_internal_metadata": obj.get("include_internal_metadata")
                if obj.get("include_internal_metadata") is not None
                else False,
                "include_model_versions": obj.get("include_model_versions")
                if obj.get("include_model_versions") is not None
                else False,
            }
        )
        return _obj



# coding: utf-8

"""
TrueFoundry ML API

No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Dict, Optional

from truefoundry.ml._autogen.client.models.command import Command
from truefoundry.pydantic_v1 import BaseModel, Field, StrictStr


class TriggerJobRunConfigRequestDto(BaseModel):
    """
    TriggerJobRunConfigRequestDto
    """

    control_plane_url: StrictStr = Field(...)
    application_fqn: StrictStr = Field(...)
    application_id: StrictStr = Field(...)
    command: Optional[Command] = None
    params: Optional[Dict[str, StrictStr]] = None
    __properties = [
        "control_plane_url",
        "application_fqn",
        "application_id",
        "command",
        "params",
    ]

    class Config:
        """Pydantic configuration"""

        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> TriggerJobRunConfigRequestDto:
        """Create an instance of TriggerJobRunConfigRequestDto from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True, exclude={}, exclude_none=True)
        # override the default output from truefoundry.pydantic_v1 by calling `to_dict()` of command
        if self.command:
            _dict["command"] = self.command.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> TriggerJobRunConfigRequestDto:
        """Create an instance of TriggerJobRunConfigRequestDto from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return TriggerJobRunConfigRequestDto.parse_obj(obj)

        _obj = TriggerJobRunConfigRequestDto.parse_obj(
            {
                "control_plane_url": obj.get("control_plane_url"),
                "application_fqn": obj.get("application_fqn"),
                "application_id": obj.get("application_id"),
                "command": Command.from_dict(obj.get("command"))
                if obj.get("command") is not None
                else None,
                "params": obj.get("params"),
            }
        )
        return _obj



import os
from tempfile import TemporaryDirectory
from typing import List, Optional

from truefoundry.deploy._autogen.models import DockerFileBuild
from truefoundry.deploy.builder.builders import dockerfile
from truefoundry.deploy.builder.builders.tfy_notebook_buildpack.dockerfile_template import (
    NotebookImageBuild,
    generate_dockerfile_content,
)
from truefoundry.deploy.builder.utils import has_python_package_manager_conf_secret

__all__ = ["generate_dockerfile_content", "build"]


def _convert_to_dockerfile_build_config(
    build_configuration: NotebookImageBuild,
    local_dir: str,
    mount_pip_conf_secret: bool = False,
) -> DockerFileBuild:
    dockerfile_content = generate_dockerfile_content(
        build_configuration=build_configuration,
        local_dir=local_dir,
        mount_pip_conf_secret=mount_pip_conf_secret,
    )
    dockerfile_path = os.path.join(local_dir, "Dockerfile")
    with open(dockerfile_path, "w", encoding="utf8") as fp:
        fp.write(dockerfile_content)
    return DockerFileBuild(
        type="dockerfile",
        dockerfile_path=dockerfile_path,
        build_context_path=local_dir,
    )


def build(
    tag: str,
    build_configuration: NotebookImageBuild,
    extra_opts: Optional[List[str]] = None,
):
    mount_pip_conf_secret = (
        has_python_package_manager_conf_secret(extra_opts) if extra_opts else False
    )
    with TemporaryDirectory() as local_dir:
        docker_build_configuration = _convert_to_dockerfile_build_config(
            build_configuration,
            local_dir=local_dir,
            mount_pip_conf_secret=mount_pip_conf_secret,
        )
        dockerfile.build(
            tag=tag,
            build_configuration=docker_build_configuration,
            extra_opts=extra_opts,
        )



import os
import time
import uuid
import zipfile
from io import BytesIO
from typing import (
    TYPE_CHECKING,
    Any,
    Dict,
    Iterator,
    List,
    Optional,
    Sequence,
    Tuple,
    Union,
)

import coolname
from truefoundry_sdk import Collaborator, MlRepoManifest, NotFoundError
from truefoundry_sdk.core import ApiError

from truefoundry import client
from truefoundry.common.utils import ContextualDirectoryManager
from truefoundry.ml import constants
from truefoundry.ml._autogen.client import (  # type: ignore[attr-defined]
    ArtifactDto,
    ArtifactType,
    CreateDatasetRequestDto,
    CreateRunRequestDto,
    DatasetDto,
    ExperimentsApi,
    ExportDeploymentFilesRequestDto,
    ListArtifactsRequestDto,
    ListArtifactVersionsRequestDto,
    ListDatasetsRequestDto,
    ListModelVersionsRequestDto,
    MlfoundryArtifactsApi,
    ModelDto,
    ModelServer,
    ModelVersionEnvironment,
    RunsApi,
    RunTagDto,
    SearchRunsRequestDto,
)
from truefoundry.ml._autogen.client.exceptions import (
    ApiException,
    NotFoundException,
)
from truefoundry.ml.enums import ModelFramework, ViewType
from truefoundry.ml.exceptions import MlFoundryException
from truefoundry.ml.internal_namespace import NAMESPACE
from truefoundry.ml.log_types.artifacts.artifact import (
    ArtifactPath,
    ArtifactVersion,
    BlobStorageDirectory,
)
from truefoundry.ml.log_types.artifacts.dataset import DataDirectory
from truefoundry.ml.log_types.artifacts.general_artifact import _log_artifact_version
from truefoundry.ml.log_types.artifacts.model import (
    ModelVersion,
    _log_model_version,
)
from truefoundry.ml.logger import logger
from truefoundry.ml.mlfoundry_run import MlFoundryRun
from truefoundry.ml.session import (
    MLFoundrySession,
    _get_api_client,
)
from truefoundry.ml.validation_utils import (
    _validate_ml_repo_description,
    _validate_ml_repo_name,
    _validate_run_name,
)

if TYPE_CHECKING:
    from truefoundry.ml import ModelFrameworkType

_SEARCH_MAX_RESULTS_DEFAULT = 1000
_ML_FOUNDRY_API_REQUEST_TIMEOUT = 10
_INTERNAL_ENV_VARS = [
    "TFY_INTERNAL_APPLICATION_ID",
    "TFY_INTERNAL_JOB_RUN_NAME",
]


def _get_internal_env_vars_values() -> Dict[str, str]:
    env = {}
    for env_var_name in _INTERNAL_ENV_VARS:
        value = os.getenv(env_var_name)
        if value:
            env[env_var_name] = value

    return env


def _resolve_version(version: Union[int, str]) -> int:
    if not isinstance(version, int) and not (
        isinstance(version, str) and version.isnumeric()
    ):
        raise MlFoundryException(
            f"version must be an integer or string containing numbers only. Got {version!r}"
        )
    final_version = int(version)
    if final_version <= 0:
        raise ValueError("version must be greater than 0")
    return final_version


class MlFoundry:
    """MlFoundry."""

    # TODO (chiragjn): Don't allow session as None here!
    def __init__(self, session: MLFoundrySession):
        """__init__

        Args:
            session (Optional[Session], optional): Session instance to get auth credentials from
        """
        self._api_client = _get_api_client(session=session)
        self._experiments_api = ExperimentsApi(api_client=self._api_client)
        self._runs_api = RunsApi(api_client=self._api_client)
        self._mlfoundry_artifacts_api = MlfoundryArtifactsApi(
            api_client=self._api_client
        )

    def _get_ml_repo_id(self, ml_repo: str) -> str:
        """_get_ml_repo_id.

        Args:
            ml_repo (str): The name of the ML Repo.

        Returns:
            str: The id of the ML Repo.
        """
        try:
            result = list(client.ml_repos.list(name=ml_repo, limit=1))
            ml_repo_instance = result[0]
        except (IndexError, NotFoundError):
            err_msg = (
                f"ML Repo Does Not Exist for name: {ml_repo}. You may either "
                f"create it from the dashboard or using "
                f"`client.create_ml_repo(ml_repo='{ml_repo}', storage_integration_fqn='<storage_integration_fqn>')`"
            )
            raise MlFoundryException(err_msg) from None
        except ApiError as e:
            err_msg = (
                f"Error happened in getting ML Repo based on name: "
                f"{ml_repo}. Error details: {e}"
            )
            raise MlFoundryException(err_msg) from e
        return ml_repo_instance.id

    def list_ml_repos(self) -> List[str]:
        """Returns a list of names of ML Repos accessible by the current user.

        Returns:
            List[str]: A list of names of ML Repos
        """
        # TODO (chiragjn): This API should yield ML Repo Entities instead of just names
        #   Kinda useless without it
        ml_repos_names = []
        max_results = 25
        try:
            for ml_repo_instance in client.ml_repos.list(limit=max_results):
                if ml_repo_instance.id != "0":
                    ml_repos_names.append(ml_repo_instance.manifest.name)
        except ApiError as e:
            err_msg = f"Error happened in fetching ML Repos. Error details: {e}"
            raise MlFoundryException(err_msg) from e
        return ml_repos_names

    def create_ml_repo(
        self,
        name: str,
        storage_integration_fqn: str,
        description: Optional[str] = None,
    ):
        """Creates an ML Repository.

        Args:
            name (str): The name of the Repository you want to create.
            storage_integration_fqn(str): The storage integration FQN to use for the experiment
                for saving artifacts.
            description (str): A description for ML Repo.

        Examples:

            ### Create Repository
            ```python
            from truefoundry.ml import get_client

            client = get_client()

            client.create_ml_repo(
                ml_repo="my-repo",
                # This controls which bucket is used.
                # You can get this from Platform > Integrations > Copy FQN of any Blob Storage type integration.
                storage_integration_fqn="..."
            )
            ```
        """
        _validate_ml_repo_name(ml_repo_name=name)
        if description:
            _validate_ml_repo_description(description=description)
        try:
            result = list(client.ml_repos.list(name=name, limit=1))
            existing_ml_repo = result[0]
        except (IndexError, NotFoundError):
            existing_ml_repo = None
        except ApiError as e:
            err_msg = (
                f"Error happened in getting ML Repo based on name: "
                f"{name}. Error details: {e}"
            )
            raise MlFoundryException(err_msg) from e

        if not existing_ml_repo:
            session = client._get_session()
            user_info = session.user_info
            if not user_info.email:
                raise MlFoundryException(
                    "Virtual accounts cannot be used to create new ML Repos"
                )
            try:
                client.ml_repos.create_or_update(
                    manifest=MlRepoManifest(
                        name=name,
                        description=description,
                        storage_integration_fqn=storage_integration_fqn,
                        collaborators=[
                            Collaborator(
                                subject=f"user:{user_info.email}",
                                role_id="mlf-project-admin",
                            )
                        ],
                    ),
                )
            except ApiError as e:
                err_msg = f"Error happened in creating ML Repo with name: {name}. Error details: {e}"
                raise MlFoundryException(err_msg) from e
            return

        if existing_ml_repo.manifest.storage_integration_fqn != storage_integration_fqn:
            raise MlFoundryException(
                f"ML Repo with same name already exists with storage integration:"
                f"{existing_ml_repo.manifest.storage_integration_fqn!r}. "
                f"Cannot update the storage integration to: {storage_integration_fqn!r}"
            )

    def create_run(
        self,
        ml_repo: str,
        run_name: Optional[str] = None,
        tags: Optional[Dict[str, Any]] = None,
        **kwargs,
    ) -> MlFoundryRun:
        """Initialize a `run`.

        In a machine learning experiment `run` represents a single experiment
        conducted under a ML Repo.
        Args:
            ml_repo (str): The name of the ML Repo under which the run will be created.
                ml_repo should only contain alphanumerics (a-z,A-Z,0-9) or hyphen (-).
                The user must have `ADMIN` or `WRITE` access to this ML Repo.
            run_name (Optional[str], optional): The name of the run. If not passed, a randomly
                generated name is assigned to the run. Under a ML Repo, all runs should have
                a unique name. If the passed `run_name` is already used under a ML Repo, the
                `run_name` will be de-duplicated by adding a suffix.
                run name should only contain alphanumerics (a-z,A-Z,0-9) or hyphen (-).
            tags (Optional[Dict[str, Any]], optional): Optional tags to attach with
                this run. Tags are key-value pairs.
            kwargs:

        Returns:
            MlFoundryRun: An instance of `MlFoundryRun` class which represents a `run`.

        Examples:

            ### Create a run under current user.
            ```python
            from truefoundry.ml import get_client

            client = get_client()

            tags = {"model_type": "svm"}
            run = client.create_run(
                ml_repo="my-classification-project", run_name="svm-with-rbf-kernel", tags=tags
            )

            run.end()
            ```

            ### Creating a run using context manager.
            ```python
            from truefoundry.ml import get_client

            client = get_client()
            with client.create_run(
                ml_repo="my-classification-project", run_name="svm-with-rbf-kernel"
            ) as run:
                # ...
                # Model training code
                ...
            # `run` will be automatically marked as `FINISHED` or `FAILED`.
            ```

            ### Create a run in a ML Repo owned by a different user.
            ```python
            from truefoundry.ml import get_client

            client = get_client()

            tags = {"model_type": "svm"}
            run = client.create_run(
                ml_repo="my-classification-project",
                run_name="svm-with-rbf-kernel",
                tags=tags,
            )
            run.end()
            ```
        """
        if not run_name:
            run_name = coolname.generate_slug(2)
            logger.info(
                f"No run_name given. Using a randomly generated name {run_name}."
                " You can pass your own using the `run_name` argument"
            )
        _validate_run_name(name=run_name)
        ml_repo_id = self._get_ml_repo_id(ml_repo=ml_repo)
        if tags is not None:
            NAMESPACE.validate_namespace_not_used(tags.keys())
        else:
            tags = {}

        tags.update(_get_internal_env_vars_values())
        _run = self._runs_api.create_run_post(
            CreateRunRequestDto(
                start_time=int(
                    time.time() * 1000
                ),  # TODO (chiragjn): computing start time should be on server side!
                experiment_id=ml_repo_id,
                name=run_name,
                tags=[RunTagDto(key=k, value=v) for k, v in tags.items()],
            )
        )
        run = _run.run
        mlf_run_id = run.info.run_id
        kwargs.setdefault("auto_end", True)
        mlf_run = MlFoundryRun(experiment_id=ml_repo_id, run_id=mlf_run_id, **kwargs)
        mlf_run._add_git_info()
        mlf_run._add_python_truefoundry_version()
        logger.info(f"Run {run.info.fqn!r} has started.")
        logger.info(f"Link to the dashboard for the run: {mlf_run.dashboard_link}")
        return mlf_run

    def get_run_by_id(self, run_id: str) -> MlFoundryRun:
        """Get an existing `run` by the `run_id`.

        Args:
            run_id (str): run_id or fqn of an existing `run`.

        Returns:
            MlFoundryRun: An instance of `MlFoundryRun` class which represents a `run`.

        Examples:

            ### Get run by the run id
            ```python
            from truefoundry.ml import get_client

            client = get_client()

            run = client.get_run_by_id(run_id='a8f6dafd70aa4baf9437a33c52d7ee90')
            ```
        """
        if run_id == "" or (not isinstance(run_id, str)):
            raise MlFoundryException(
                f"run_id must be string type and not empty. "
                f"Got {type(run_id)} type with value {run_id}"
            )
        if "/" in run_id:
            return self.get_run_by_fqn(run_id)
        _run = self._runs_api.get_run_get(run_id=run_id)
        run = _run.run
        mlfoundry_run = MlFoundryRun._from_dto(run)
        logger.info(
            f"Link to the dashboard for the run: {mlfoundry_run.dashboard_link}"
        )
        return mlfoundry_run

    def get_run_by_fqn(self, run_fqn: str) -> MlFoundryRun:
        """Get an existing `run` by `fqn`.

        `fqn` stands for Fully Qualified Name. A run `fqn` has the following pattern:
        tenant_name/ml_repo/run_name

        If  a run `svm` under the ML Repo `cat-classifier` in `truefoundry` tenant,
        the `fqn` will be `truefoundry/cat-classifier/svm`.

        Args:
            run_fqn (str): `fqn` of an existing run.

        Returns:
            MlFoundryRun: An instance of `MlFoundryRun` class which represents a `run`.

        Examples:

            ### get run by run fqn
            ```python
            from truefoundry.ml import get_client

            client = get_client()

            run = client.get_run_by_fqn(run_fqn='truefoundry/my-repo/svm')
            ```
        """
        _run = self._runs_api.get_run_by_fqn_get(run_fqn=run_fqn)
        run = _run.run
        mlfoundry_run = MlFoundryRun._from_dto(run)
        logger.info(
            f"Link to the dashboard for the run: {mlfoundry_run.dashboard_link}"
        )
        return mlfoundry_run

    def get_run_by_name(
        self,
        ml_repo: str,
        run_name: str,
    ) -> MlFoundryRun:
        """Get an existing `run` by `run_name`.

        Args:
            ml_repo (str): name of the ml_repo of which the run is part of.
            run_name (str): the name of the run required

        Returns:
            MlFoundryRun: An instance of `MlFoundryRun` class which represents a `run`.

        Examples:

            ### get run by name
            ```python
            from truefoundry.ml import get_client

            client = get_client()

            run = client.get_run_by_name(run_name='svm', ml_repo='my-repo')
            ```
        """
        _run = self._runs_api.get_run_by_name_get(
            experiment_id=None,
            run_name=run_name,
            experiment_name=ml_repo,
        )
        run = _run.run
        mlfoundry_run = MlFoundryRun._from_dto(run)
        logger.info(
            f"Link to the dashboard for the run: {mlfoundry_run.dashboard_link}"
        )
        return mlfoundry_run

    def search_runs(
        self,
        ml_repo: str,
        filter_string: str = "",
        run_view_type: ViewType = ViewType.ACTIVE_ONLY.value,
        order_by: Sequence[str] = ("attribute.start_time DESC",),
        job_run_name: Optional[str] = None,
    ) -> Iterator[MlFoundryRun]:
        """
        The user must have `READ` access to the ML Repo.
        Returns an iterator that returns a MLFoundryRun on each next call.
        All the runs under a ML Repo which matches the filter string and the run_view_type are returned.

        Args:
            ml_repo (str): Name of the ML Repo.
            filter_string (str, optional):
                Filter query string, defaults to searching all runs.
                Identifier required in the LHS of a search expression.
                Signifies an entity to compare against. An identifier has two parts separated by a period: the
                type of the entity and the name of the entity.
                The type of the entity is metrics, params, attributes, or tags.
                The entity name can contain alphanumeric characters and special characters.
                You can search using two run attributes : status and artifact_uri. Both attributes have string values.
                When a metric, parameter, or tag name contains a special character like hyphen, space, period,
                and so on, enclose the entity name in double quotes or backticks,
                params."model-type" or params.`model-type`

            run_view_type (str, optional): one of the following values "ACTIVE_ONLY", "DELETED_ONLY", or "ALL" runs.
            order_by (List[str], optional):
                List of columns to order by (e.g., "metrics.rmse"). Currently supported values
                are metric.key, parameter.key, tag.key, attribute.key. The ``order_by`` column
                can contain an optional ``DESC`` or ``ASC`` value. The default is ``ASC``.
                The default ordering is to sort by ``start_time DESC``.

            job_run_name (str): Name of the job which are associated with the runs to get that runs

        Returns:
            Iterator[MlFoundryRun]: MLFoundryRuns matching the search query.

        Examples:

            ```python
            from truefoundry.ml import get_client

            client = get_client()
            with client.create_run(ml_repo="my-project", run_name="run-1") as run1:
                run1.log_metrics(metric_dict={"accuracy": 0.74, "loss": 0.6})
                run1.log_params({"model": "LogisticRegression", "lambda": "0.001"})

            with client.create_run(ml_repo="my-project", run_name="run-2") as run2:
                run2.log_metrics(metric_dict={"accuracy": 0.8, "loss": 0.4})
                run2.log_params({"model": "SVM"})

            # Search for the subset of runs with logged accuracy metric greater than 0.75
            filter_string = "metrics.accuracy > 0.75"
            runs = client.search_runs(ml_repo="my-project", filter_string=filter_string)

            # Search for the subset of runs with logged accuracy metric greater than 0.7
            filter_string = "metrics.accuracy > 0.7"
            runs = client.search_runs(ml_repo="my-project", filter_string=filter_string)

            # Search for the subset of runs with logged accuracy metric greater than 0.7 and model="LogisticRegression"
            filter_string = "metrics.accuracy > 0.7 and params.model = 'LogisticRegression'"
            runs = client.search_runs(ml_repo="my-project", filter_string=filter_string)

            # Search for the subset of runs with logged accuracy metric greater than 0.7 and
            # order by accuracy in Descending order
            filter_string = "metrics.accuracy > 0.7"
            order_by = ["metric.accuracy DESC"]
            runs = client.search_runs(
                ml_repo="my-project", filter_string=filter_string, order_by=order_by
            )

            filter_string = "metrics.accuracy > 0.7"
            runs = client.search_runs(
                ml_repo="transformers", order_by=order_by ,job_run_name='job_run_name', filter_string=filter_string
            )

            order_by = ["metric.accuracy DESC"]
            runs = client.search_runs(
                ml_repo="my-project", filter_string=filter_string, order_by=order_by, max_results=10
            )
            ```
        """
        _validate_ml_repo_name(ml_repo_name=ml_repo)
        try:
            result = list(client.ml_repos.list(name=ml_repo, limit=1))
            ml_repo_instance = result[0]
        except (IndexError, NotFoundError):
            raise MlFoundryException(
                f"ML Repo with name {ml_repo} does not exist. "
                f"You may either create it from the dashboard or using "
                f"`client.create_ml_repo(ml_repo='{ml_repo}', storage_integration_fqn='<storage_integration_fqn>')`"
            ) from None
        except ApiError as e:
            raise MlFoundryException(
                f"ML Repo with name {ml_repo} does not exist or your user does not have permission to access it: {e}"
            ) from e

        ml_repo_id = ml_repo_instance.id

        page_token = None
        done = False
        if job_run_name:
            if filter_string == "":
                filter_string = f"tags.TFY_INTERNAL_JOB_RUN_NAME = '{job_run_name}'"
            else:
                filter_string += (
                    f" and tags.TFY_INTERNAL_JOB_RUN_NAME = '{job_run_name}'"
                )
        while not done:
            runs_page = self._runs_api.search_runs_post(
                SearchRunsRequestDto(
                    experiment_ids=[ml_repo_id],
                    filter=filter_string,
                    run_view_type=run_view_type,
                    max_results=_SEARCH_MAX_RESULTS_DEFAULT,
                    order_by=order_by,
                    page_token=page_token,
                )
            )
            runs = runs_page.runs
            page_token = runs_page.next_page_token

            for run in runs:
                yield MlFoundryRun._from_dto(run)
            if not runs or page_token is None:
                done = True

    def _initialize_model_server(
        self,
        name: str,
        model_version_fqn: str,
        workspace_fqn: str,
        model_server: ModelServer,
        output_dir: Optional[str] = None,
    ) -> str:
        """
        Initialize the model server for deployment.

        Args:
            name (str): Name of the application.
            model_version_fqn (str): Fully Qualified Name of the model version.
            workspace_fqn (str): Fully Qualified Name of the workspace.
            model_server (ModelServer): Server type for deployment (e.g., TRITON).
            output_dir (Optional[str]): Directory where the model files will be extracted.
                                        Defaults to the current working directory.

        Returns:
            str: Path to the directory where the model files are extracted.

        Raises:
            MlFoundryException: If an error occurs during API calls, directory creation, or file extraction.
        """

        # Using get_with_http_info to get the response object and extract the raw data
        try:
            export_deployment_files_request_dto = ExportDeploymentFilesRequestDto(
                model_version_fqn=model_version_fqn,
                workspace_fqn=workspace_fqn,
                service_name=name,
                model_server=model_server,
            )
            response = self._mlfoundry_artifacts_api.export_deployment_files_by_fqn_post_with_http_info(
                export_deployment_files_request_dto=export_deployment_files_request_dto,
                _preload_content=False,
                _request_timeout=_ML_FOUNDRY_API_REQUEST_TIMEOUT,
            )
        except ApiException as e:
            err_msg = (
                f"Failed to fetch deployment files for name={name}, "
                f"model_version_fqn={model_version_fqn}, workspace_fqn={workspace_fqn}. "
                f"Error: {e}"
            )
            raise MlFoundryException(err_msg) from e

        output_dir = os.path.abspath(output_dir or os.getcwd())
        codegen_dir = os.path.join(output_dir, name)
        if codegen_dir == output_dir:
            raise ValueError("Name cannot be empty, please provide a valid name")

        try:
            with ContextualDirectoryManager(dir_path=codegen_dir) as dir_path:
                with zipfile.ZipFile(BytesIO(response.raw_data), mode="r") as zip_file:
                    zip_file.extractall(dir_path)
        except FileExistsError as e:
            err_msg = (
                f"Deployment directory {codegen_dir!r} already exists. "
                "Please choose a different deployment name or delete the existing directory."
            )
            raise MlFoundryException(err_msg) from e
        except zipfile.BadZipFile as e:
            raise MlFoundryException(
                f"Failed to extract model files. Error: {e}"
            ) from e
        return codegen_dir

    def get_model_version(
        self,
        ml_repo: str,
        name: str,
        version: Union[str, int] = constants.LATEST_ARTIFACT_OR_MODEL_VERSION,
    ) -> Optional[ModelVersion]:
        """
        Get the model version to download contents or load it in memory

        Args:
            ml_repo (str): ML Repo to which model is logged
            name (str): Model Name
            version (str | int): Model Version to fetch (default is the latest version)

        Returns:
           ModelVersion: The ModelVersion instance of the model.

        Examples:

            ### Sklearn

            ```python
            # See `truefoundry.ml.mlfoundry_api.MlFoundry.log_model` examples to understand model logging
            import tempfile
            import joblib
            from truefoundry.ml import get_client

            client = get_client()
            model_version = client.get_model_version(
                ml_repo="my-classification-project",
                name="my-sklearn-model",
                version=1
            )

            # Download the model to disk
            temp = tempfile.TemporaryDirectory()
            download_info = model_version.download(path=temp.name)
            print(download_info.model_dir, download_info.model_filename)

            # Deserialize and Load
            model = joblib.load(
                os.path.join(download_info.model_dir, download_info.model_filename)
            )
            ```

            ### Huggingface Transformers

            ```python
            # See `truefoundry.ml.mlfoundry_api.MlFoundry.log_model` examples to understand model logging
            import torch
            from transformers import pipeline

            from truefoundry.ml import get_client

            client = get_client()
            model_version = client.get_model_version(
                ml_repo="my-llm-project",
                name="my-transformers-model",
                version=1
            )

            # Download the model to disk
            temp = tempfile.TemporaryDirectory()
            download_info = model_version.download(path=temp.name)
            print(download_info.model_dir)

            # Deserialize and Load
            pln = pipeline("text-generation", model=download_info.model_dir, torch_dtype=torch.float16)
            ```
        """
        resolved_version = None
        if version != constants.LATEST_ARTIFACT_OR_MODEL_VERSION:
            resolved_version = _resolve_version(version=version)

        ml_repo_id = self._get_ml_repo_id(ml_repo=ml_repo)

        _model_version = self._mlfoundry_artifacts_api.get_model_version_by_name_get(
            experiment_id=int(ml_repo_id),
            model_name=name,
            version=resolved_version,
        )
        model_version = _model_version.model_version
        _model = self._mlfoundry_artifacts_api.get_model_get(id=model_version.model_id)
        model = _model.model

        return ModelVersion(
            model_version=model_version,
            model=model,
        )

    def get_model_version_by_fqn(self, fqn: str) -> ModelVersion:
        """
        Get the model version to download contents or load it in memory

        Args:
            fqn (str): Fully qualified name of the model version.

        Returns:
            ModelVersion: The ModelVersion instance of the model.

        Examples:

            ### Sklearn

            ```python
            # See `truefoundry.ml.mlfoundry_api.MlFoundry.log_model` examples to understand model logging
            import tempfile
            import joblib
            from truefoundry.ml import get_client

            client = get_client()
            model_version = client.get_model_version_by_fqn(
                fqn="model:truefoundry/my-classification-project/my-sklearn-model:1"
            )

            # Download the model to disk
            temp = tempfile.TemporaryDirectory()
            download_info = model_version.download(path=temp.name)
            print(download_info.model_dir, download_info.model_filename)

            # Deserialize and Load
            model = joblib.load(
                os.path.join(download_info.model_dir, download_info.model_filename)
            )
            ```

            ### Huggingface Transformers

            ```python
            # See `truefoundry.ml.mlfoundry_api.MlFoundry.log_model` examples to understand model logging
            import torch
            from transformers import pipeline

            from truefoundry.ml import get_client

            client = get_client()
            model_version = client.get_model_version_by_fqn(
                fqn="model:truefoundry/my-llm-project/my-transformers-model:1"
            )
            # Download the model to disk
            temp = tempfile.TemporaryDirectory()
            download_info = model_version.download(path=temp.name)
            print(download_info.model_dir)

            # Deserialize and Load
            pln = pipeline("text-generation", model=download_info.model_dir, torch_dtype=torch.float16)
            ```
        """
        return ModelVersion.from_fqn(fqn=fqn)

    def list_model_versions(self, ml_repo: str, name: str) -> Iterator[ModelVersion]:
        """
        Get all the version of a model to download contents or load them in memory

        Args:
            ml_repo (str): Repository in which the model is stored.
            name (str): Name of the model whose version is required

        Returns:
            Iterator[ModelVersion]: An iterator that yields non deleted model versions
                of a model under a given ml_repo  sorted reverse by the version number

        Examples:

            ```python
            from truefoundry.ml import get_client

            client = get_client()
            model_versions = client.list_model_version(ml_repo="my-repo", name="svm")

            for model_version in model_versions:
                print(model_version)
            ```
        """
        ml_repo_id = self._get_ml_repo_id(ml_repo=ml_repo)
        try:
            _model = self._mlfoundry_artifacts_api.get_model_by_name_get(
                experiment_id=int(ml_repo_id), name=name
            )
        except NotFoundException as e:
            err_msg = (
                f"Model Does Not Exist for ml_repo={ml_repo}, name={name}. Error: {e}"
            )
            raise MlFoundryException(err_msg) from e
        model = _model.model
        return self._list_model_versions_by_id(model=model)

    def list_model_versions_by_fqn(self, model_fqn: str) -> Iterator[ModelVersion]:
        """
        List versions for a given model

        Args:
            model_fqn: FQN of the Model to list versions for.
                A model_fqn looks like `model:{org}/{user}/{project}/{artifact_name}`
                or `model:{user}/{project}/{artifact_name}`

        Returns:
            Iterator[ModelVersion]: An iterator that yields non deleted model versions
                under the given model_fqn sorted reverse by the version number

        Yields:
            ModelVersion: An instance of `truefoundry.ml.ModelVersion`

        Examples:

            ```python
            from truefoundry.ml import get_client

            client = get_client()
            model_fqn = "model:org/my-project/my-model"
            for mv in client.list_model_versions(model_fqn=model_fqn):
                print(mv.name, mv.version, mv.description)
            ```
        """
        _model = self._mlfoundry_artifacts_api.get_model_by_fqn_get(fqn=model_fqn)
        model = _model.model
        return self._list_model_versions_by_id(model=model)

    def _list_model_versions_by_id(
        self,
        model_id: Optional[uuid.UUID] = None,
        model: Optional[ModelDto] = None,
    ) -> Iterator[ModelVersion]:
        if model and not model_id:
            model_id = model.id
        elif not model and model_id:
            _model = self._mlfoundry_artifacts_api.get_model_get(id=str(model_id))
            model = _model.model
        else:
            raise MlFoundryException(
                "Exactly one of model_id or model should be passed"
            )

        max_results, page_token, done = 10, None, False
        while not done:
            _model_versions = self._mlfoundry_artifacts_api.list_model_versions_post(
                list_model_versions_request_dto=ListModelVersionsRequestDto(
                    model_id=str(model_id),
                    max_results=max_results,
                    page_token=page_token,
                )
            )
            model_versions = _model_versions.model_versions
            page_token = _model_versions.next_page_token
            for model_version in model_versions:
                yield ModelVersion(model_version=model_version, model=model)
            if not model_versions or not page_token:
                done = True

    def get_artifact_version(
        self,
        ml_repo: str,
        name: str,
        artifact_type: Optional[ArtifactType] = ArtifactType.ARTIFACT,
        version: Union[str, int] = constants.LATEST_ARTIFACT_OR_MODEL_VERSION,
    ) -> Optional[ArtifactVersion]:
        """
        Get the model version to download contents or load it in memory

        Args:
            ml_repo (str): ML Repo to which artifact is logged
            name (str): Artifact Name
            artifact_type (str): The type of artifact to fetch (acceptable values: "artifact", "model", "plot", "image")
            version (str | int): Artifact Version to fetch (default is the latest version)

        Returns:
            ArtifactVersion : An ArtifactVersion instance of the artifact

        Examples:

            ```python
            import tempfile
            from truefoundry.ml import get_client

            client = get_client()
            artifact_version = client.get_artifact_version(ml_repo="ml-repo-name", name="artifact-name", version=1)

            # download the artifact to disk
            temp = tempfile.TemporaryDirectory()
            download_path = artifact_version.download(path=temp.name)
            print(download_path)
            ```
        """
        resolved_version = None
        if version != constants.LATEST_ARTIFACT_OR_MODEL_VERSION:
            resolved_version = _resolve_version(version=version)

        ml_repo_id = self._get_ml_repo_id(ml_repo=ml_repo)

        _artifact_version = (
            self._mlfoundry_artifacts_api.get_artifact_version_by_name_get(
                experiment_id=int(ml_repo_id),
                artifact_name=name,
                artifact_type=artifact_type,
                version=resolved_version,
            )
        )
        artifact_version = _artifact_version.artifact_version
        _artifact = self._mlfoundry_artifacts_api.get_artifact_by_id_get(
            id=artifact_version.artifact_id
        )
        artifact = _artifact.artifact

        return ArtifactVersion(
            artifact_version=artifact_version,
            artifact=artifact,
        )

    def get_artifact_version_by_fqn(self, fqn: str) -> ArtifactVersion:
        """
        Get the artifact version to download contents

        Args:
            fqn (str): Fully qualified name of the artifact version.

        Returns:
            ArtifactVersion : An ArtifactVersion instance of the artifact

        Examples:

            ```python
            import tempfile
            from truefoundry.ml import get_client

            client = get_client()
            artifact_version = client.get_artifact_version_by_fqn(
                fqn="artifact:truefoundry/my-classification-project/sklearn-artifact:1"
            )

            # download the artifact to disk
            temp = tempfile.TemporaryDirectory()
            download_path = artifact_version.download(path=temp.name)
            print(download_path)
            ```
        """
        return ArtifactVersion.from_fqn(fqn=fqn)

    def list_artifact_versions(
        self,
        ml_repo: str,
        name: str,
        artifact_type: Optional[ArtifactType] = ArtifactType.ARTIFACT,
    ) -> Iterator[ArtifactVersion]:
        """
        Get all the version of na artifact to download contents or load them in memory

        Args:
            ml_repo (str): Repository in which the model is stored.
            name (str): Name of the artifact whose version is required
            artifact_type (ArtifactType): Type of artifact you want for example model, image, etc.

        Returns:
            Iterator[ArtifactVersion]: An iterator that yields non deleted artifact-versions
                of an artifact under a given ml_repo  sorted reverse by the version number

        Examples:

            ```python
            from truefoundry.ml import get_client

            client = get_client()
            artifact_versions = client.list_artifact_versions(ml_repo="my-repo", name="artifact-name")

            for artifact_version in artifact_versions:
                print(artifact_version)
            ```
        """
        ml_repo_id = self._get_ml_repo_id(ml_repo=ml_repo)
        _artifacts = self._mlfoundry_artifacts_api.list_artifacts_post(
            list_artifacts_request_dto=ListArtifactsRequestDto(
                experiment_id=ml_repo_id,
                name=name,
                artifact_types=[artifact_type] if artifact_type else None,
                max_results=1,
            )
        )
        artifacts = _artifacts.artifacts
        if not artifacts or len(artifacts) == 0:
            err_msg = f"Artifact Does Not Exist for ml_repo={ml_repo}, name={name}, type={artifact_type}"
            raise MlFoundryException(err_msg)
        return self._list_artifact_versions_by_id(artifact=artifacts[0])

    def list_artifact_versions_by_fqn(
        self, artifact_fqn: str
    ) -> Iterator[ArtifactVersion]:
        """
        List versions for a given artifact

        Args:
            artifact_fqn: FQN of the Artifact to list versions for.
                An artifact_fqn looks like `{artifact_type}:{org}/{user}/{project}/{artifact_name}`
                or `{artifact_type}:{user}/{project}/{artifact_name}`

                where artifact_type can be on of ("model", "image", "plot")

        Returns:
            Iterator[ArtifactVersion]: An iterator that yields non deleted artifact versions
                under the given artifact_fqn sorted reverse by the version number

        Yields:
            ArtifactVersion: An instance of `truefoundry.ml.ArtifactVersion`

        Examples:

            ```python
            from truefoundry.ml import get_client

            client = get_client()
            artifact_fqn = "artifact:org/my-project/my-artifact"
            for av in client.list_artifact_versions(artifact_fqn=artifact_fqn):
                print(av.name, av.version, av.description)
            ```
        """

        _artifact = self._mlfoundry_artifacts_api.get_artifact_by_fqn_get(
            fqn=artifact_fqn
        )
        artifact = _artifact.artifact
        return self._list_artifact_versions_by_id(artifact=artifact)

    def _list_artifact_versions_by_id(
        self,
        artifact_id: Optional[uuid.UUID] = None,
        artifact: Optional[ArtifactDto] = None,
    ) -> Iterator[ArtifactVersion]:
        if artifact and not artifact_id:
            artifact_id = artifact.id
        elif not artifact and artifact_id:
            _artifact = self._mlfoundry_artifacts_api.get_artifact_by_id_get(
                id=str(artifact_id)
            )
            artifact = _artifact.artifact
        else:
            raise MlFoundryException(
                "Exactly one of artifact_id or artifact should be passed"
            )

        max_results, page_token, done = 10, None, False
        while not done:
            _artifact_versions = (
                self._mlfoundry_artifacts_api.list_artifact_versions_post(
                    list_artifact_versions_request_dto=ListArtifactVersionsRequestDto(
                        artifact_id=str(artifact_id),
                        max_results=max_results,
                        page_token=page_token,
                    )
                )
            )
            artifact_versions = _artifact_versions.artifact_versions
            page_token = _artifact_versions.next_page_token
            for artifact_version in artifact_versions:
                yield ArtifactVersion(
                    artifact_version=artifact_version, artifact=artifact
                )
            if not artifact_versions or not page_token:
                done = True

    def log_artifact(
        self,
        ml_repo: str,
        name: str,
        artifact_paths: List[
            Union[Tuple[str], Tuple[str, Optional[str]], ArtifactPath]
        ],
        description: Optional[str] = None,
        metadata: Optional[Dict[str, Any]] = None,
        progress: Optional[bool] = None,
    ) -> Optional[ArtifactVersion]:
        """Logs an artifact for the current `ml_repo`.

        An `artifact` is a list of local files and directories.
        This function packs the mentioned files and directories in `artifact_paths`
        and uploads them to remote storage linked to the ml_repo

        Args:
            ml_repo (str): Name of the ML Repo to which an artifact is to be logged.
            name (str): Name of the Artifact. If an artifact with this name already exists under the current ml_repo,
                the logged artifact will be added as a new version under that `name`. If no artifact exist with
                the given `name`, the given artifact will be logged as version 1.
            artifact_paths (List[truefoundry.ml.ArtifactPath], optional): A list of pairs
                of (source path, destination path) to add files and folders
                to the artifact version contents. The first member of the pair should be a file or directory path
                and the second member should be the path inside the artifact contents to upload to.
            progress (bool): value to show progress bar, defaults to None.

                ```python

                from truefoundry.ml import get_client, ArtifactPath

                client = get_client()
                client.log_artifact(
                    ml_repo="sample-repo",
                    name="xyz",
                    artifact_paths=[
                        ArtifactPath("foo.txt", "foo/bar/foo.txt"),
                        ArtifactPath("tokenizer/", "foo/tokenizer/"),
                        ArtifactPath('bar.text'),
                        ('bar.txt', ),
                        ('foo.txt', 'a/foo.txt')
                    ]
                )
                ```

                would result in

                ```
                .
                └── foo/
                    ├── bar/
                    │   └── foo.txt
                    └── tokenizer/
                        └── # contents of tokenizer/ directory will be uploaded here
                ```
            description (Optional[str], optional): arbitrary text upto 1024 characters to store as description.
                This field can be updated at any time after logging. Defaults to `None`
            metadata (Optional[Dict[str, Any]], optional): arbitrary json serializable dictionary to store metadata.
                For example, you can use this to store metrics, params, notes.
                This field can be updated at any time after logging. Defaults to `None`

        Returns:
            truefoundry.ml.ArtifactVersion: an instance of `ArtifactVersion` that can be used to download the files,
            or update attributes like description, metadata.

        Examples:

            ```python
            import os
            from truefoundry.ml import get_client, ArtifactPath

            with open("artifact.txt", "w") as f:
                f.write("hello-world")

            client = get_client()
            ml_repo = "sample-repo"

            client.create_ml_repo(
                ml_repo=ml_repo,
                # This controls which bucket is used.
                # You can get this from Platform > Integrations > Copy FQN of any Blob Storage type integration.
                storage_integration_fqn="..."
            )
            client.log_artifact(
                ml_repo=ml_repo,
                name="hello-world-file",
                artifact_paths=[ArtifactPath('artifact.txt', 'a/b/')]
            )
            ```
        """
        if not artifact_paths:
            raise MlFoundryException(
                "artifact_paths cannot be empty, atleast one artifact_path must be passed"
            )
        artifact_version = _log_artifact_version(
            run=None,
            ml_repo=ml_repo,
            name=name,
            artifact_paths=artifact_paths,
            description=description,
            metadata=metadata,
            step=None,
            progress=progress,
        )
        logger.info(f"Logged artifact successfully with fqn {artifact_version.fqn!r}")
        return artifact_version

    def log_model(
        self,
        *,
        ml_repo: str,
        name: str,
        model_file_or_folder: Union[str, BlobStorageDirectory],
        description: Optional[str] = None,
        metadata: Optional[Dict[str, Any]] = None,
        progress: Optional[bool] = None,
        framework: Optional[Union[str, ModelFramework, "ModelFrameworkType"]] = None,
        environment: Optional[ModelVersionEnvironment] = None,
    ) -> ModelVersion:
        """
        Serialize and log a versioned model under the current ml_repo. Each logged model generates a new version
        associated with the given `name` and linked to the current run. Multiple versions of the model can be
        logged as separate versions under the same `name`.

        Args:
            ml_repo (str): Name of the ML Repo to which an artifact is to be logged.
            name (str): Name of the model. If a model with this name already exists under the current ML Repo,
                the logged model will be added as a new version under that `name`. If no models exist with the given
                `name`, the given model will be logged as version 1.

            model_file_or_folder (Union[str, BlobStorageDirectory]):
                str:
                    Path to either a single file or a folder containing model files.
                    This folder is typically created using serialization methods from libraries or frameworks,
                    e.g., `joblib.dump`, `model.save_pretrained(...)`, `torch.save(...)`, or `model.save(...)`.
                BlobStorageDirectory:
                    uri (str): URI to the model file or folder in a storage integration associated with the specified ML Repo.
                        The model files or folder must reside within the same storage integration as the specified ML Repo.
                        Accepted URI formats include `s3://integration-bucket-name/prefix/path/to/model` or `gs://integration-bucket-name/prefix/path/to/model`.
                        If the URI points to a model in a different storage integration, an error will be raised indicating "Invalid source URI."

            framework (Optional[Union[ModelFramework, ModelFrameworkType]]): Framework used for model serialization.
                Supported frameworks values (ModelFrameworkType) can be imported from `from truefoundry.ml import *`.
                Supported frameworks can be found in `truefoundry.ml.enums.ModelFramework`.
                Can also be `None` if the framework is not known or not supported.
                **Deprecated**: Prefer `ModelFrameworkType` over `enums.ModelFramework`.

            description (Optional[str], optional): arbitrary text upto 1024 characters to store as description.
                This field can be updated at any time after logging. Defaults to `None`
            metadata (Optional[Dict[str, Any]], optional): arbitrary json serializable dictionary to store metadata.
                For example, you can use this to store metrics, params, notes.
                This field can be updated at any time after logging. Defaults to `None`
            progress (bool): value to show progress bar, defaults to None.

        Returns:
            truefoundry.ml.ModelVersion: an instance of `ModelVersion` that can be used to download the files,
                load the model, or update attributes like description, metadata, schema.

        Examples:

            ### Sklearn

            ```python
            from truefoundry.ml import get_client, SklearnFramework

            import joblib
            import numpy as np
            from sklearn.pipeline import make_pipeline
            from sklearn.preprocessing import StandardScaler
            from sklearn.svm import SVC

            X = np.array([[-1, -1], [-2, -1], [1, 1], [2, 1]])
            y = np.array([1, 1, 2, 2])
            clf = make_pipeline(StandardScaler(), SVC(gamma='auto'))
            clf.fit(X, y)
            joblib.dump(clf, "sklearn-pipeline.joblib")

            client = get_client()
            client.create_ml_repo(  # This is only required once
                ml_repo="my-classification-project",
                # This controls which bucket is used.
                # You can get this from Platform > Integrations > Copy FQN of any Blob Storage type integration.
                storage_integration_fqn="..."
            )
            model_version = client.log_model(
                ml_repo="my-classification-project",
                name="my-sklearn-model",
                model_file_or_folder="sklearn-pipeline.joblib",
                framework=SklearnFramework(),
                metadata={"accuracy": 0.99, "f1": 0.80},
                step=1,  # step number, useful when using iterative algorithms like SGD
            )
            print(model_version.fqn)
            ```

            ### Huggingface Transformers

            ```python
            from truefoundry.ml import get_client, TransformersFramework, LibraryName

            import torch
            from transformers import AutoTokenizer, AutoConfig, pipeline, AutoModelForCausalLM
            pln = pipeline(
                "text-generation",
                model_file_or_folder="EleutherAI/pythia-70m",
                tokenizer="EleutherAI/pythia-70m",
                torch_dtype=torch.float16
            )
            pln.model.save_pretrained("my-transformers-model")
            pln.tokenizer.save_pretrained("my-transformers-model")

            client = get_client()
            client.create_ml_repo(  # This is only required once
                ml_repo="my-llm-project",
                # This controls which bucket is used.
                # You can get this from Integrations > Blob Storage. `None` picks the default
                storage_integration_fqn=None
            )
            model_version = client.log_model(
                ml_repo="my-llm-project",
                name="my-transformers-model",
                model_file_or_folder="my-transformers-model/",
                framework=TransformersFramework(library_name=LibraryName.TRANSFORMERS, pipeline_tag='text-generation')
            )
            print(model_version.fqn)
            ```

        """
        model_version = _log_model_version(
            run=None,
            ml_repo=ml_repo,
            name=name,
            model_file_or_folder=model_file_or_folder,
            description=description,
            metadata=metadata,
            step=None,
            progress=progress,
            framework=framework,
            environment=environment,
        )
        logger.info(f"Logged model successfully with fqn {model_version.fqn!r}")
        return model_version

    # Datasets API
    def create_data_directory(
        self,
        ml_repo: str,
        name: str,
        description: Optional[str] = None,
        metadata: Optional[Dict[str, Any]] = None,
    ) -> DataDirectory:
        """
        Create DataDirectory to Upload the files

        Args:
            ml_repo (str): Name of the ML Repo in which you want to create data_directory
            name (str): Name of the DataDirectory to be created.
            description (str): Description for the DataDirectory.
            metadata (Dict <str>: Any): Metadata about the data_directory in Dictionary form.

        Returns:
            DataDirectory : An instance of class DataDirectory

        Examples:

            ```python
            from truefoundry.ml import get_client

            client = get_client()
            data_directory = client.create_data_directory(name="<data_directory-name>", ml_repo="<repo-name>")
            print(data_directory.fqn)
            ```
        """
        if name == "" or not isinstance(name, str):
            raise MlFoundryException(
                f"DataDirectory name must be string type and not empty. "
                f"Got {type(name)} type with value {name}"
            )

        if ml_repo == "" or not isinstance(ml_repo, str):
            raise MlFoundryException(
                f"ML repo must be string type and not empty. "
                f"Got {type(ml_repo)} type with value {ml_repo}"
            )

        ml_repo_id = self._get_ml_repo_id(ml_repo=ml_repo)
        # TODO: Add get_data_directory_by_name on server
        _datasets = self._mlfoundry_artifacts_api.list_datasets_post(
            list_datasets_request_dto=ListDatasetsRequestDto(
                experiment_id=ml_repo_id, name=name, max_results=1
            )
        )
        datasets = _datasets.datasets
        if datasets is not None and len(datasets) > 0:
            logger.warning(
                f"Data Directory with the name {name} already exists in ML Repo {ml_repo}, "
                f"returning the original instance of DataDirectory instead"
            )
            return DataDirectory(dataset=datasets[0])

        _dataset = self._mlfoundry_artifacts_api.create_dataset_post(
            create_dataset_request_dto=CreateDatasetRequestDto(
                name=name,
                experiment_id=ml_repo_id,
                description=description,
                dataset_metadata=metadata,
            )
        )
        dataset = _dataset.dataset
        return DataDirectory(dataset=dataset)

    def get_data_directory_by_fqn(
        self,
        fqn: str,
    ) -> DataDirectory:
        """
        Get the DataDirectory by DataDirectory FQN

        Args:
            fqn (str): Fully qualified name of the artifact version.

        Returns:
            DataDirectory : An instance of class DataDirectory

        Examples:

            ```python
            from truefoundry.ml import get_client, DataDirectoryPath

            client = get_client()
            data_directory = client.get_data_directory_by_fqn(fqn="<data-dir-fqn>")
            with open("artifact.txt", "w") as f:
                f.write("hello-world")

            data_directory.add_files(
                artifact_paths=[DataDirectoryPath('artifact.txt', 'a/b/')]
            )
            # print the path of files and folder in the data_directory
            for file in data_directory.list_files():
                print(file.path)
            ```
        """

        _dataset = self._mlfoundry_artifacts_api.get_dataset_by_fqn_get(fqn=fqn)
        dataset = _dataset.dataset
        return DataDirectory(dataset)

    def get_data_directory(
        self,
        ml_repo: str,
        name: str,
    ) -> DataDirectory:
        """Get an existing `data_directory` by `name`.
        Args:
            ml_repo (str): name of the ML Repo the data-directory is part of.
            name (str): the name of the data-directory
        Returns:
            DataDirectory: An instance of class DataDirectory
        Examples:
            ```python
            from truefoundry.ml import get_client
            client = get_client()
            data_directory = client.get_data_directory(ml_repo='my-repo', name="<data-directory-name>")
            with open("artifact.txt", "w") as f:
                f.write("hello-world")
            data_directory.add_files(
                artifact_paths=[DataDirectoryPath('artifact.txt', 'a/b/')]
            )
            # print the path of files and folder in the data_directory
            for file in data_directory.list_files():
                print(file.path)
            ```
        """
        if ml_repo == "" or not isinstance(ml_repo, str):
            raise MlFoundryException(
                f"ML repo must be string type and not empty. "
                f"Got {type(ml_repo)} type with value {ml_repo}"
            )
        ml_repo_id = self._get_ml_repo_id(ml_repo=ml_repo)
        _datasets = self._mlfoundry_artifacts_api.list_datasets_post(
            list_datasets_request_dto=ListDatasetsRequestDto(
                experiment_id=ml_repo_id,
                name=name,
                max_results=1,
            ),
        )
        datasets = _datasets.datasets
        if not datasets or len(datasets) == 0:
            raise MlFoundryException(
                f"No data directory found with name {name} under ML Repo {ml_repo}"
            )

        return DataDirectory(dataset=datasets[0])

    def list_data_directories(
        self,
        ml_repo: str,
    ) -> Iterator[DataDirectory]:
        """
        Get the list of DataDirectory in a ml_repo

        Args:
            ml_repo (str): Name of the ML Repository

        Returns:
            DataDirectory : An instance of class DataDirectory

        Examples:

            ```python
            from truefoundry.ml import get_client

            client = get_client()
            data_directories = client.list_data_directories(ml_repo="<ml-repo-nam>")

            for data_directory in data_directories:
                print(data_directory.name)
            ```
        """
        if ml_repo == "" or not isinstance(ml_repo, str):
            raise MlFoundryException(
                f"ML repo must be string type and not empty. "
                f"Got {type(ml_repo)} type with value {ml_repo}"
            )
        ml_repo_id = self._get_ml_repo_id(ml_repo=ml_repo)
        max_results, page_token, done = 10, None, False
        while not done:
            _datasets = self._mlfoundry_artifacts_api.list_datasets_post(
                list_datasets_request_dto=ListDatasetsRequestDto(
                    experiment_id=ml_repo_id,
                    max_results=max_results,
                    page_token=page_token,
                )
            )
            datasets: List[DatasetDto] = _datasets.datasets or []
            page_token = _datasets.next_page_token
            for dataset in datasets:
                yield DataDirectory(dataset=dataset)
            if not datasets or not page_token:
                done = True


def get_client() -> MlFoundry:
    """Initializes and returns the mlfoundry client.


    Returns:
        MlFoundry: Instance of `MlFoundry` class which represents a `run`.

    Examples:

        ### Get client
        ```python
        from truefoundry.ml import get_client

        client = get_client()
        ```
    """
    session = MLFoundrySession.new()
    return MlFoundry(session=session)



from __future__ import annotations

import enum
from typing import List, Optional

from openai import OpenAI
from rich.console import Console

from truefoundry.autodeploy.agents.base import Agent
from truefoundry.autodeploy.tools import (
    Ask,
    FileTypeCounts,
    ListFiles,
    ReadFile,
    RequestEvent,
    ResponseEvent,
)
from truefoundry.pydantic_v1 import Field


class ComponentType(str, enum.Enum):
    SERVICE = "SERVICE"
    JOB = "JOB"


class ProjectIdentifier(Agent):
    description = """
Identify the type of project.
"""
    system_prompt = """
Your goal is to figure out the type of project.
Once you figure out, send the Response tool call.
The information will be later used to build a Dockerfile.
Try to figure out ALL the response tool field arguments even if something is optional.
Prefer reading files like requirements.txt, pyproject.toml, go.mod, packages.json to figure out framework.
You should have strong justification for your response. Do not assume anything.
Prefer using file type counts over list files.
use the knowledge of file type counts to create the right glob pattern while listing files.
Prefer yarn over npm if the project is using yarn.
If there is  a lock file, the project maybe using yarn.
Look for *.lock if there is a lock type file present.
Always response with a function call.
    """
    max_iter = 10

    class Request(RequestEvent): ...

    class Response(ResponseEvent):
        component_type: Optional[ComponentType] = Field(
            None,
            description="""
A Service is designed to run always and should never terminate.
A Job is designed to finish after sometime.
""",
        )
        primary_programming_language: str = Field(
            ...,
            description="""
Primary programming language used for the project.
Ex: Go, Python, Rust, Typescript, etc.",
""",
        )
        framework: Optional[str] = Field(
            None,
            description="""
If the project is using any specific framework.
Ex: FastAPI, Gin, Flask, NestJS, React, etc.
            """,
        )
        version: Optional[str] = Field(
            None,
            description="""
Identifies and return the exact version of the project's programming language,
essential for successful Docker image creation and execution.""",
        )
        dependency_files: Optional[List[str]] = Field(
            None,
            descripton="""
requirements.txt, poetry.lock, yarn.lock, Cargo.lock, go.mod, go.sum, pyproject.toml, etc.
There can be multiple files like ["pyproject.toml", "poetry.lock"] or ["yarn.lock", "package.json"]
                                                  """,
        )
        dependency_manager: Optional[str] = Field(
            None,
            descripton="""
pip, poetry, yarn, go.mod, cargo.toml, npm, setup.py.
                                                  """,
        )
        justification: str = Field(
            ...,
            description="Justification behind each response field.",
        )

        def render(self, console: Console):
            console.rule("[bold green]Project Identification[/]")
            if self.primary_programming_language is not None:
                console.print(
                    f"[bold magenta]TrueFoundry:[/] Identified a project using [bold cyan]{self.primary_programming_language}[/]."
                )
                console.print(
                    f"[bold magenta]TrueFoundry:[/] Framework Identified: [bold cyan]{'Not applicable' if self.framework is None else self.framework}[/]"
                )
                console.print(
                    f"[bold magenta]TrueFoundry:[/] Dependency Manager Identified: [bold cyan]{'Not applicable' if self.dependency_manager is None else self.dependency_manager}[/]"
                )
            else:
                console.print(
                    "[bold magenta]TrueFoundry:[/] Unable to identify any programming language in the project."
                )
            console.print(
                f"[bold magenta]TrueFoundry:[/] [italic]{self.justification}[/]"
            )

    def __init__(self, project_root_path: str, openai_client: OpenAI):
        self.tools = [
            ReadFile(
                project_root_path=project_root_path,
            ),
            ListFiles(
                project_root_path=project_root_path,
            ),
            FileTypeCounts(project_root_path=project_root_path),
            Ask(),
        ]
        self.openai_client = openai_client



from __future__ import annotations

import itertools
import re
from typing import Any, Generator, Iterable, Optional

import docker
from docker.models.images import BuildError, json_stream
from rich.console import Console, ConsoleOptions, RenderResult
from rich.padding import Padding
from rich.text import Text

from truefoundry.autodeploy.tools.base import (
    Event,
    Message,
    RequestEvent,
    ResponseEvent,
    Rule,
    Tool,
)
from truefoundry.pydantic_v1 import Field


class DockerBuildLog(Event):
    log: str

    def render(self, console: Console):
        console.print(Padding.indent(renderable=Text.from_ansi(self.log), level=2))


# vendored from
# https://github.com/docker/docker-py/blob/9ad4bddc9ee23f3646f256280a21ef86274e39bc/docker/models/images.py#L220
def _build(docker_client: docker.DockerClient, **kwargs) -> Iterable[DockerBuildLog]:
    resp = docker_client.images.client.api.build(**kwargs)
    if isinstance(resp, str):
        return docker_client.images.get(resp)
    last_event = None
    image_id = None
    result_stream, internal_stream = itertools.tee(json_stream(resp))
    for chunk in internal_stream:
        if "error" in chunk:
            raise BuildError(chunk["error"], result_stream)
        if "stream" in chunk:
            yield DockerBuildLog(log=chunk["stream"])
            match = re.search(
                r"(^Successfully built |sha256:)([0-9a-f]+)$", chunk["stream"]
            )
            if match:
                image_id = match.group(2)
        last_event = chunk
    if image_id:
        return None
    raise BuildError(last_event or "Unknown", result_stream)


class DockerBuild(Tool):
    description = """
Build a docker image.
    """

    class Request(RequestEvent):
        dockerfile_path: str = Field(
            ...,
            pattern=r"^[a-zA-Z0-9\.]{1}.*$",
            description="Dockerfile path. ",
        )
        image_tag: str = Field(..., description="image tag")

    class Response(ResponseEvent):
        error: Optional[str] = Field(None, description="Error raised while building")
        build_logs: Optional[str] = Field(None, description="Build logs")

        def __rich_console__(
            self, console: Console, options: ConsoleOptions
        ) -> RenderResult:
            none_text = "[italic magenta]None[/]"
            error_text = (
                f"[green]'{self.error}'[/]" if self.error is not None else none_text
            )
            yield Text.from_markup("[bold magenta]Response[/](")
            if self.build_logs is not None:
                yield Text.from_markup('  [yellow]build_logs[/]= "')
                yield Text.from_ansi(self.build_logs)
                yield Text.from_markup('"')
            else:
                yield Text.from_markup(f"  [yellow]build_logs[/]={none_text}")
            yield Text.from_markup(f"  [yellow]error[/]={error_text}\n)")

    def __init__(self, project_root_path: str, docker_client: docker.DockerClient):
        self.project_root_path = project_root_path
        self.docker_client = docker_client

    def run(self, request: DockerBuild.Request) -> Generator[Event, Any, ResponseEvent]:
        yield Rule(message="[bold green] Docker Build[/]")
        yield Message(message="[bold cyan]Processing:[/] Building Docker image...")
        yield Message(message="[bold yellow]Docker build logs:[/]")
        try:
            for message in _build(
                self.docker_client,
                path=self.project_root_path,
                tag=request.image_tag,
            ):
                yield message
            return DockerBuild.Response()
        except BuildError as ex:
            logs = ""
            for log_line in ex.build_log:
                logs += log_line.get("stream", "")
            return DockerBuild.Response(error=str(ex), build_logs=logs[-800:])
        except (docker.errors.APIError, docker.errors.DockerException) as ex:
            return DockerBuild.Response(error=str(ex), build_logs="")



# coding: utf-8

"""
TrueFoundry ML API

No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Optional

from truefoundry.ml._autogen.client.models.library_name import LibraryName
from truefoundry.pydantic_v1 import BaseModel, Field, StrictStr, validator


class TransformersFramework(BaseModel):
    """
    TransformersFramework
    """

    type: StrictStr = Field(default=..., description="Type of the framework")
    library_name: Optional[LibraryName] = Field(
        default=None, description="Name of the library for the framework"
    )
    pipeline_tag: Optional[StrictStr] = Field(
        default=None,
        description="The `pipeline()` task this model can be used with e.g. `text-generation`. See [huggingface docs](https://huggingface.co/docs/transformers/main/en/main_classes/pipelines#transformers.pipeline.task) for all possible values",
    )
    base_model: Optional[StrictStr] = Field(
        default=None,
        description="Base model Id from HuggingFace Hub. If this is a finetuned model, this points to the base model id used for finetuning.",
    )
    __properties = ["type", "library_name", "pipeline_tag", "base_model"]

    @validator("type")
    def type_validate_enum(cls, value):
        """Validates the enum"""
        if value not in ("transformers",):
            raise ValueError("must be one of enum values ('transformers')")
        return value

    class Config:
        """Pydantic configuration"""

        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> TransformersFramework:
        """Create an instance of TransformersFramework from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True, exclude={}, exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> TransformersFramework:
        """Create an instance of TransformersFramework from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return TransformersFramework.parse_obj(obj)

        _obj = TransformersFramework.parse_obj(
            {
                "type": obj.get("type"),
                "library_name": obj.get("library_name"),
                "pipeline_tag": obj.get("pipeline_tag"),
                "base_model": obj.get("base_model"),
            }
        )
        return _obj



from typing import Callable, Dict

from truefoundry.deploy.builder.builders import (
    dockerfile,
    tfy_notebook_buildpack,
    tfy_python_buildpack,
    tfy_spark_buildpack,
    tfy_task_pyspark_buildpack,
)

BUILD_REGISTRY: Dict[str, Callable] = {
    "dockerfile": dockerfile.build,
    "tfy-python-buildpack": tfy_python_buildpack.build,
    "tfy-notebook-buildpack": tfy_notebook_buildpack.build,
    "tfy-spark-buildpack": tfy_spark_buildpack.build,
    "task-pyspark-build": tfy_task_pyspark_buildpack.build,
}

__all__ = ["get_builder"]


def get_builder(build_configuration_type: str) -> Callable:
    if build_configuration_type not in BUILD_REGISTRY:
        raise NotImplementedError(f"Builder for {build_configuration_type} not found")

    return BUILD_REGISTRY[build_configuration_type]



# coding: utf-8

"""
TrueFoundry ML API

No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, Dict, Optional

from truefoundry.ml._autogen.client.models.parameters import Parameters
from truefoundry.pydantic_v1 import BaseModel, Field, StrictStr, constr, validator


class ModelConfiguration(BaseModel):
    """
    Add a model from Gateway along with parameters to be used for chat completion  # noqa: E501
    """

    provider: StrictStr = Field(
        default=...,
        description="Name of the provider, must be one of the integration providers configured for the Gateway",
    )
    model: constr(strict=True) = Field(
        default=...,
        description="Name of the model to be used for generations. This model should be available in the provider",
    )
    parameters: Optional[Parameters] = Field(
        default=None, description="Parameters to pass to the model when generating"
    )
    extra_parameters: Optional[Dict[str, Any]] = Field(
        default=None,
        description="Arbitrary extra parameters to pass to the model when generating",
    )
    __properties = ["provider", "model", "parameters", "extra_parameters"]

    @validator("model")
    def model_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if not re.match(r"^[a-zA-Z][a-zA-Z0-9\-]{1,30}[a-zA-Z0-9]$", value):
            raise ValueError(
                r"must validate the regular expression /^[a-zA-Z][a-zA-Z0-9\-]{1,30}[a-zA-Z0-9]$/"
            )
        return value

    class Config:
        """Pydantic configuration"""

        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> ModelConfiguration:
        """Create an instance of ModelConfiguration from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True, exclude={}, exclude_none=True)
        # override the default output from truefoundry.pydantic_v1 by calling `to_dict()` of parameters
        if self.parameters:
            _dict["parameters"] = self.parameters.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> ModelConfiguration:
        """Create an instance of ModelConfiguration from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return ModelConfiguration.parse_obj(obj)

        _obj = ModelConfiguration.parse_obj(
            {
                "provider": obj.get("provider"),
                "model": obj.get("model"),
                "parameters": Parameters.from_dict(obj.get("parameters"))
                if obj.get("parameters") is not None
                else None,
                "extra_parameters": obj.get("extra_parameters"),
            }
        )
        return _obj



from __future__ import annotations

import atexit
from typing import Any, Dict, Generator, List, Optional

import docker
import requests
from rich.console import Console, ConsoleOptions, RenderResult
from rich.padding import Padding
from rich.text import Text

from truefoundry.autodeploy.tools.base import (
    Event,
    Message,
    RequestEvent,
    ResponseEvent,
    Rule,
    Tool,
)
from truefoundry.pydantic_v1 import Field


class DockerRunLog(Event):
    log: str

    def render(self, console: Console):
        console.print(Padding.indent(renderable=Text.from_ansi(self.log), level=2))


class DockerRun(Tool):
    description = """
Run a docker image
"""

    class Request(RequestEvent):
        image_tag: str
        ports: Optional[Dict[str, int]] = Field(
            None,
            description="""
Ports to expose.
The keys of the dictionary are the ports to bind inside the container in 'port'.
The values are the ports to open on the host""",
        )
        command: str

        def render(self, console: Console):
            console.print(
                f"[bold magenta]TrueFoundry[/] is executing the Docker container. Image Tag: [bold green]{self.image_tag}[/], Exposed Port: [bold green]{str(self.ports) if self.ports is not None else 'Not exposed'}[/], Command: [bold green]{self.command}[/]"
            )

    class Response(ResponseEvent):
        logs: Optional[List[str]] = Field(
            None, description="Logs of the container. Only last 50 chars."
        )
        exit_code: Optional[int] = Field(
            None,
            description="""
Exit code of the process if the container stops.
This will not be passed if the container is still running.
""",
        )
        client_error: Optional[str] = Field(None, description="Docker client error.")

        def __rich_console__(
            self, console: Console, options: ConsoleOptions
        ) -> RenderResult:
            none_text = "[italic magenta]None[/]"
            error_text = (
                f"[green]'{self.client_error}'[/]"
                if self.client_error is not None
                else none_text
            )
            exit_code_text = (
                "[green]'0'[/]"
                if self.exit_code == 0
                else (
                    none_text if self.exit_code is None else f"[red]{self.exit_code}[/]"
                )
            )

            yield Text.from_markup("[bold magenta]Response[/](")
            if self.logs is not None:
                yield Text.from_markup('  [yellow]logs=[/]"')
                yield Text.from_ansi("".join(self.logs))
                yield Text.from_markup('"')
            else:
                yield Text.from_markup(f"  [yellow]logs[/]={none_text}")

            yield Text.from_markup(f"  [yellow]exit_code[/]={exit_code_text}")
            yield Text.from_markup(f"  [yellow]client_error[/]={error_text} \n)")

    def __init__(self, docker_client: docker.DockerClient, environment: Dict):
        self.containers = []
        self.docker_client = docker_client
        self.environment = environment
        atexit.register(self._kill_running_containers)

    def _kill_running_containers(self):
        if self.containers:
            container = self.containers.pop()
            try:
                container.remove(force=True)
            except docker.errors.APIError:
                pass

    def run(self, request: DockerRun.Request) -> Generator[Event, Any, ResponseEvent]:
        self._kill_running_containers()

        yield Rule(message="[bold green] Docker Run[/]")
        yield Message(message="[bold cyan]Testing:[/] Running Docker container...")
        try:
            container = self.docker_client.containers.run(
                request.image_tag,
                detach=True,
                remove=False,
                stderr=True,
                ports=request.ports,
                environment=self.environment,
                command=request.command,
            )
        except docker.errors.APIError as ex:
            if ex.is_client_error():
                return DockerRun.Response(client_error=str(ex))
            raise
        yield Message(message="[bold yellow]Docker logs:[/]")
        self.containers.append(container)
        exit_code = None

        all_logs = []

        try:
            # This acts as a signal to update status.
            yield DockerRunLog(log="")
            logs = container.logs(stream=True)
            for log in logs:
                log = log.decode()
                all_logs.append(log)
                yield DockerRunLog(log=log)
        except KeyboardInterrupt:
            pass
        else:
            yield Message(message="\n[bold yellow]There are no more logs.[/]")
        exit_code = None
        try:
            exit_code = container.wait(timeout=1).get("StatusCode")
        except (
            requests.exceptions.ReadTimeout,
            requests.exceptions.ConnectionError,
        ):
            ...
        return DockerRun.Response(logs=all_logs, exit_code=exit_code)
